package no.jabber.eclipseplugin;

import java.io.File;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collection;

import no.jabber.eclipseplugin.jdtsearch.JdtUtil;
import no.jabber.eclipseplugin.linking.EclipseLink;
import no.jabber.eclipseplugin.linking.IEclipseLinkParamsTypes;
import no.jabber.eclipseplugin.linking.IFileTypes;
import no.jabber.eclipseplugin.linking.LinkParameter;
import no.jabber.eclipseplugin.linking.UnsupportedJavaElement;
import no.jabber.eclipseplugin.projectmanagement.Project;
import no.jabber.eclipseplugin.projectmanagement.ProjectConfig;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * Utility class with  various methods for performing operations pertaining to the GUI
 */
public class GuiUtil
{
	
	
	public static GridData createFillGridData()
	{
		GridData grd = new GridData();
		grd.grabExcessHorizontalSpace = true;
		grd.grabExcessVerticalSpace = true;
		grd.horizontalAlignment = SWT.FILL;
		grd.verticalAlignment = SWT.FILL;
		return grd;
	}
	
	public static Font boldifyFont(boolean bMakeBold, Font fntOrig)
	{
		String strFontName = fntOrig.getFontData()[0].getName();
		int nHeight = fntOrig.getFontData()[0].getHeight();
		return new Font(Display.getDefault(), new FontData(strFontName,nHeight,bMakeBold ? SWT.BOLD  : SWT.NORMAL));
	}
	
	private static boolean checkExistence(String strRelativePath)
	{
		Path pth = new Path(strRelativePath);
		// Check if file really exists on disk
		Path pthTest = new Path(GuiUtil.getCurrentWorkspace().getRoot().getLocation().toOSString());
		pthTest = (Path)pthTest.append(pth);
		File fileTest = pthTest.toFile();
		return (fileTest.exists()) && (fileTest.isFile());
	}
	/**
	 * Opens the correct editor for a eclipse link at the location specified in the link
	 * @param eclLink The link to open
	 */
	public static void openEditorForEclipseLink(EclipseLink eclLink) throws CoreException, BadLocationException
	{
		IPath pth = Path.fromPortableString(eclLink.getPath());
		boolean bExists = checkExistence(pth.toPortableString());
		if(!bExists) 
		{
			IPath ptsh = Path.fromPortableString(eclLink.getPath());
			String strCommonNameCandidate = ptsh.segment(0);
			if(strCommonNameCandidate != null)
			{
				Collection<Project> prjs = ProjectConfig.getConfig().getProjects();
				for (Project project : prjs)
				{
					if(project.getCommonName().equalsIgnoreCase(strCommonNameCandidate))
					{
						IPath p = Path.fromOSString(project.getLocalMapping());
						String strLocalPrjNameCandidate = p.segment(p.segmentCount()-1);
						IProject prj = GuiUtil.getCurrentWorkspace().getRoot().getProject(strLocalPrjNameCandidate);
						if(prj != null)
						{
							
							pth = prj.getFullPath().append(pth.removeFirstSegments(1));
							bExists = checkExistence(pth.toPortableString());
							break;
						}
					}
				}
			}
			
		}
		if (bExists)
		{
			IFile file = GuiUtil.getCurrentWorkspace().getRoot().getFile(pth);
			if (file.isLocal(IFile.DEPTH_ZERO))
			{
				// Find the correct editor type
				IEditorDescriptor edtDesc = GuiUtil.getCurrentWorkbench().getEditorRegistry().getDefaultEditor(eclLink.getPath());
				FileEditorInput edtInput = new FileEditorInput(file);
				IEditorPart edt = null;
				edt = GuiUtil.getCurrentWorkbench().getActiveWorkbenchWindow().getActivePage().openEditor(edtInput, edtDesc.getId());
				if (edt != null)
				{
					if (eclLink.getFileType().equalsIgnoreCase(IFileTypes.FT_JAVA))
					{
						// Link is a java file
						if ((eclLink.getParameters() != null) && (eclLink.getParameters().size() > 0) )
						{
							// It has parameters
							if (edt instanceof ITextEditor)
							{
								IJavaElement clFile = (IJavaElement)edtInput.getAdapter(IJavaElement.class);
								if (clFile != null)
								{
									// Find where the parameters point to
									String strMethod = null;
									LinkParameter lpMth = eclLink.getParameter(IEclipseLinkParamsTypes.METHOD_NAME);
									if (lpMth != null) strMethod = lpMth.getParamValue();
									String strField = null;
									LinkParameter lpFld = eclLink.getParameter(IEclipseLinkParamsTypes.FIELD_NAME);
									if (lpFld != null) strField = lpFld.getParamValue();
									ArrayList<SearchMatch> alMatches = null;
									if (strMethod != null)
									{

										alMatches = JdtUtil.performJdtSearch(clFile, strMethod, IJavaSearchConstants.METHOD);
									}
									else if (strField != null)
									{
										alMatches = JdtUtil.performJdtSearch(clFile, strField, IJavaSearchConstants.FIELD);
									}
									else
									{
										String strLineNumber = eclLink.getParameter(IEclipseLinkParamsTypes.LINE_NUMBER).getParamValue();
										if (strLineNumber != null) GuiUtil.jumpToLine(strLineNumber, edt, edtInput);
									}
									if (alMatches != null)
									{
										if (alMatches.size() > 0)
										{
											SearchMatch matchFirst = alMatches.get(0);
											JavaUI.revealInEditor(edt, (IJavaElement)matchFirst.getElement());
											// Add link to MRU list
//											SimpleChatView.instance.addToEclipseLinkMRU(eclLink);
										}
									}
								}
							}
						}
						else
						{
							// no parameters
//							SimpleChatView.instance.addToEclipseLinkMRU(eclLink);
						}
					}
					else if ((eclLink.getFileType().equalsIgnoreCase(IFileTypes.FT_TEXT)) || (eclLink.getFileType().equalsIgnoreCase(IFileTypes.FT_UNKNOWN)))
					{
						// we have a text file or an unknown type
						if ((eclLink.getParameters() != null) && (eclLink.getParameters().size() > 0))
						{
							if (edt instanceof ITextEditor)
							{
								// Only line number is valid parameter
								String strLineNumber = eclLink.getParameter(IEclipseLinkParamsTypes.LINE_NUMBER).getParamValue();
								if (strLineNumber != null) GuiUtil.jumpToLine(strLineNumber, edt, edtInput);
//								SimpleChatView.instance.addToEclipseLinkMRU(eclLink);
							}
						}
						else
						{
							// No parameters
//							SimpleChatView.instance.addToEclipseLinkMRU(eclLink);
						}
					}
				}
			}
		}
	}
	
	/**
	 * Reveals a specificed line number in an editor
	 */
	private static void jumpToLine(String strLineNumber, IEditorPart edt, FileEditorInput edtInput) throws BadLocationException
	{
		int nLine = Integer.parseInt(strLineNumber) - 1;
		if (nLine > -1)
		{
			ITextEditor edtTxt = (ITextEditor)edt;
			int nOffset = edtTxt.getDocumentProvider().getDocument(edtInput).getLineOffset(nLine);
			edtTxt.setHighlightRange(nOffset, 0, true);
		}
	}
	
	public static Shell getCurrentShell()
	{
//		if (SimpleChatView.instance != null) return SimpleChatView.instance.getSite().getShell();
		return Display.getCurrent().getActiveShell();
//		else return null;
	}
	
	/**
	 * Convenience method for handling exceptions. Shows a dialog and prints exception to the Error Log.
	 * @param e
	 */
	@SuppressWarnings({"unused","deprecation"})
	public static void handleException(Exception e)
	{
		Shell sh = GuiUtil.getCurrentShell();
		if(sh != null)
		{
			MessageDialog.openError(sh,"An exception occured","Please check error log for debug information");
		}
		SimpleChatPlugin.getDefault().getLog().log(new Status(IStatus.ERROR, SimpleChatPlugin.getDefault().getDescriptor().getUniqueIdentifier(), IStatus.ERROR, "Error SimpleChatPlugin",e));
	}
	
	@SuppressWarnings({"unused","deprecation"})
	public static void handleException(Exception e, String strCustomMsg)
	{
		Shell sh = GuiUtil.getCurrentShell();
		if(sh != null)
		{
			MessageDialog.openError(sh,"An exception occured. ", strCustomMsg + ". Please check error log for debug information");
		}
		SimpleChatPlugin.getDefault().getLog().log(new Status(IStatus.ERROR, SimpleChatPlugin.getDefault().getDescriptor().getUniqueIdentifier(), IStatus.ERROR, "Error SimpleChatPlugin",e));
	}
	
	@SuppressWarnings({"unused","deprecation"})
	public static void handleUnsupportedJavaElementException(UnsupportedJavaElement e)
	{
		int elemType = e.getJavaElement().getElementType();
		String strElemType = "unknown";
		switch(elemType)
		{
			case IJavaElement.CLASS_FILE: strElemType = "CLASS";
			case IJavaElement.COMPILATION_UNIT: strElemType = "COMPILATION_UNIT";
			case IJavaElement.FIELD: strElemType = "FIELD";
			case IJavaElement.IMPORT_CONTAINER: strElemType = "IMPORT_CONTAINER";
			case IJavaElement.IMPORT_DECLARATION: strElemType = "IMPORT_DECLARATION";
			case IJavaElement.INITIALIZER: strElemType = "INITIALIZER";
			case IJavaElement.JAVA_MODEL: strElemType = "JAVA_MODEL";
			case IJavaElement.JAVA_PROJECT: strElemType = "JAVA_PROJECT";
			case IJavaElement.LOCAL_VARIABLE: strElemType = "LOCAL_VARIABLE";
			case IJavaElement.METHOD: strElemType = "METHOD";
			case IJavaElement.PACKAGE_DECLARATION: strElemType = "PACKAGE_DECLARATION";
			case IJavaElement.PACKAGE_FRAGMENT: strElemType = "PACKAGE_FRAGMENT";
			case IJavaElement.PACKAGE_FRAGMENT_ROOT: strElemType = "PACKAGE_FRAGMENT_ROOT";
			case IJavaElement.TYPE: strElemType = "TYPE";
			case IJavaElement.TYPE_PARAMETER: strElemType = "TYPE_PARAMETER";
		}
		GuiUtil.handleException(e, "Java element type "+ strElemType + " not supported for linking. Currently only methods and fields are supported. Consider linking to line number instead");
	}
	
	public static IWorkspace getCurrentWorkspace()
	{
		return ResourcesPlugin.getWorkspace();
	}

	public static IProject getProjectFromCurrentEditor()
	{
		return getFileFromCurrentEditor().getProject();
	}

	public static IEditorPart getCurrentEditor()
	{
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
	}

	public static IWorkbench getCurrentWorkbench()
	{
		return PlatformUI.getWorkbench();
	}

	public static IFile getFileFromCurrentEditor()
	{
		return ((IFileEditorInput)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor().getEditorInput()).getFile();
	}

	public static FormData createFormData(int lp, int lo, int rp, int ro, int tp, int to, int bp, int bo)
	{
		FormData data = new FormData();
		if (lp != -1) data.left = new FormAttachment(lp, 100, lo);
		if (rp != -1) data.right = new FormAttachment(rp, 100, ro);
		if (tp != -1) data.top = new FormAttachment(tp, 100, to);
		if (bp != -1) data.bottom = new FormAttachment(bp, 100, bo);
		return data;
	}
	

	public static IJavaElement findJavaElementMatch(String strSelection)
	{
		IJavaElement clFile = (IJavaElement)GuiUtil.getCurrentEditor().getEditorInput().getAdapter(IJavaElement.class);
		if (clFile != null)
		{
			ArrayList<SearchMatch> matches = null;
			try
			{
				matches = JdtUtil.performJdtSearch(clFile, strSelection, IJavaSearchConstants.METHOD);
				if (matches.size() == 0) matches = JdtUtil.performJdtSearch(clFile, strSelection, IJavaSearchConstants.FIELD);
			}
			catch(CoreException ex)
			{
				GuiUtil.handleException(ex);
			}
			if (matches.size() > 0) 
			{
				return (IJavaElement)matches.get(0).getElement();
			}
			return null;
		}
		return null;
	}





	public static EclipseLink createLinkEilertFromFile(IFile file, Project pr)
	{
		if (file != null)
		{
			String strPath = file.getFullPath().toString();
			EclipseLink eclLink = null;
			try
			{
				eclLink = new EclipseLink(strPath, null, pr);
			}
			catch(URISyntaxException ex)
			{
				ex.printStackTrace();
			}
			return eclLink;
		} else return null;
	}

	public static void createLinkFromFileAndInsert(IFile file, Project pr)
	{
		EclipseLink eclLink = createLinkEilertFromFile(file, pr);
		if (eclLink != null)
		{
			SimpleChatPlugin.getDefault().RequestPasteEclipseLink.fire(eclLink);
		}
	}
	
	/**
	 * Creates a eclispe link from a java element and inserts the link into the chat input textbox 
	 * @throws URISyntaxException 
	 * @throws UnsupportedJavaElement 
	 */
	public static void createLinkAndInsert(IJavaElement elem) throws URISyntaxException, UnsupportedJavaElement
	{
		EclipseLink eclLink = EclipseLink.createFromJavaElement(elem);
		if (eclLink != null)
		{
			SimpleChatPlugin.getDefault().RequestPasteEclipseLink.fire(eclLink);
		}
	}

}
