package no.jabber.eclipseplugin;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;

import no.jabber.eclipseplugin.Util.CustomEvent;
import no.jabber.eclipseplugin.Util.CustomListener;
import no.jabber.eclipseplugin.awareness.AwarenessService;
import no.jabber.eclipseplugin.linking.EclipseLink;
import no.jabber.eclipseplugin.linking.IEclipseLinkParamsTypes;
import no.jabber.eclipseplugin.linking.IFileTypes;
import no.jabber.eclipseplugin.linking.UnsupportedJavaElement;
import no.jabber.eclipseplugin.preferences.PreferenceConstants;
import no.jabber.eclipseplugin.projectmanagement.Contact;
import no.jabber.eclipseplugin.projectmanagement.Project;
import no.jabber.eclipseplugin.projectmanagement.ProjectConfig;
import no.jabber.eclipseplugin.resources.ImageCollection;
import no.jabber.eclipseplugin.serialization.AwarenessEntry;
import no.jabber.eclipseplugin.serialization.LogData;
import no.jabber.eclipseplugin.serialization.LogSession;

import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.plugin.*;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import org.jivesoftware.smack.*;
import org.jivesoftware.smack.filter.PacketTypeFilter;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;

/**
 * This singleton class is required by the framework
 * It contains metods for creating and closing the connection to the chat server
 */
public class SimpleChatPlugin extends AbstractUIPlugin
{
	private static SimpleChatPlugin plugin;
	public static String BTNCONNECT_CONNECT = "connect";
	public static String BTNCONNECT_DISCONNECT = "disconnect";
	public static final String AWARENESS_MSG_ID = "awareness_msg";

	public static final String CONTEXT_PROPERTY_ID = "context";
	public static final String CONTEXT_LOCATION_PROPERTY_ID = "location";
	public static final String MORE_INFO_PROPERTY_ID = "more_info";
	public static final String PROJECT_PROPERTY_ID = "project";
	
	private XMPPConnection con = null;

	public Hashtable<String, LogData> htLogDataObjs = new Hashtable<String, LogData>();
	public Util.CustomEvent<PresenceChangedData> PresenceChanged = new CustomEvent<PresenceChangedData>();
	public Util.CustomEvent<String> Connected = new CustomEvent<String>(); 
	public Util.CustomEvent<String> Disconnected = new CustomEvent<String>();
	public Util.CustomEvent<String> RosterChanged = new CustomEvent<String>();
	public Util.CustomEvent<ActivateViewData> ActivateChatView = new CustomEvent<ActivateViewData>();
	public Util.CustomEvent<NewContactOnRosterData> NewContactOnRoster = new CustomEvent<NewContactOnRosterData>();
	public Util.CustomEvent<EclipseLink> RequestPasteEclipseLink = new CustomEvent<EclipseLink>();
	
	private AwarenessService logAwareness;
	private Hashtable<String, AwarenessEntry> htLatestAwareness = new Hashtable<String, AwarenessEntry>();
	
//	private SimpleChatRoster roster;
//	private SimpleChatView view;
	
	private LogSession sessionActive;
	
	public void setContextFromMenu(String strContext)
	{
		if(sessionActive != null) sessionActive.setContext(strContext);
	}

	public void setContextLocationFromMenu(IPath pthLoc)
	{
		if(sessionActive != null) sessionActive.setContextLocation(pthLoc.toPortableString());
	}

	public void setContextLocationFromMenu(IJavaElement elem) throws URISyntaxException, UnsupportedJavaElement
	{
		EclipseLink ec = EclipseLink.createFromJavaElement(elem);
		if( (sessionActive != null) && (ec != null)) sessionActive.setContextLocation(ec.getLinkAsString());
	}

	public void setMoreInfoFromMenu(String strMoreInfo)
	{
		if(sessionActive != null) sessionActive.setMoreInfo(strMoreInfo);
	}
	
	
//	public void setRoster(SimpleChatRoster r)
//	{
//		assert(r != null);
//		this.roster = r;
////		this.RosterCreated.fire(r);
//	
//	}
	
	public void setView(SimpleChatView v)
	{
//		assert(v != null);
//		this.view = v;
	}
	
	public SimpleChatPlugin()
	{
		plugin = this;
//		 Maps valid parameters for each file type. Used in conjuction with
		// eclipse-links
		Util.FTYPE_PARAM_MAPPING.put(IFileTypes.FT_JAVA,IEclipseLinkParamsTypes.JAVA_PARAMS);
		Util.FTYPE_PARAM_MAPPING.put(IFileTypes.FT_TEXT,IEclipseLinkParamsTypes.TEXT_PARAMS);
		Util.FTYPE_PARAM_MAPPING.put(IFileTypes.FT_UNKNOWN,IEclipseLinkParamsTypes.TEXT_PARAMS);
	}

	private void initGfx()
	{
//		this.getImageRegistry().put("online", new Image(d,d));
		ImageCollection c = ImageCollection.getInstance();
		c.setPath("gfx");
		c.loadResource("online", "online2.gif");
		c.loadResource("offline", "offline2.gif");
		c.loadResource("connect", "online.gif");
		c.loadResource("icoRoster", "contacts.gif");
		c.loadResource("icoLinkBuilder", "links4.gif");
		c.loadResource("icoProjects", "projects.gif");
		c.loadResource("icoProject", "project.gif");
		c.loadResource("icoConfigure", "configure.gif");
		c.loadResource("icoLatest", "latest.gif");
		c.loadResource("icoEvents", "events.gif");
		c.loadResource("icoConv", "conv.gif");
		c.loadResource("icoConvActive", "conv_active.gif");
		c.loadResource("icoToBottom", "bottom.gif");
	}
	
	public void start(BundleContext context) throws Exception
	{
		super.start(context);
		this.initGfx();
		if (logAwareness == null) try
		{
			logAwareness = new AwarenessService();
		}
		catch (Exception ex)
		{
			GuiUtil.handleException(ex);
		}
		
		this.Disconnected.addListener(new CustomListener<String>(){
			public void onAction(String data)
			{
				for(LogData ld : htLogDataObjs.values())
				{
					ld.setContactNode(null);
				}
			}});
	}

	public void stop(BundleContext context) throws Exception
	{
		super.stop(context);
		for(LogData ld : htLogDataObjs.values())
		{
			ld.dispose(true);
		}
		plugin = null;
		logAwareness.dispose();
	}

	public void initConnection() throws XMPPException
	{
		try
		{
			//XMPPConnection.DEBUG_ENABLED = true;
			Chat.setFilteredOnThreadID(false);
			Roster.setDefaultSubscriptionMode(Roster.SUBSCRIPTION_ACCEPT_ALL);
	
			IPreferenceStore prefs = this.getPreferenceStore();
//			boolean bGoogleTalk = prefs.getBoolean(PreferenceConstants.P_GOOGLETALK);
//			if (bGoogleTalk)
//			{
//				MessageDialog.openInformation(null, "Trbbel...", "GoogleTalk funker ikke med denne versjonen av Smack!");
//				throw new RuntimeException("GoogleTalk not supported");
//	//			con = new GoogleTalkConnection();
//			}
//			else
//			{
				String strServer = prefs.getString(PreferenceConstants.P_SERVER);
				String strPort = prefs.getString(PreferenceConstants.P_PORT);
				int nPort = Integer.parseInt(strPort);
				con = new XMPPConnection(strServer,nPort);
//			}
	
			String strUser = prefs.getString(PreferenceConstants.P_USER);
			String strPass = prefs.getString(PreferenceConstants.P_PASS);
			con.login(strUser, strPass);
	
	//		 Listens to chat messages
			con.addPacketListener(new PacketListener()
			{
				public void processPacket(Packet p)
				{
					handleMessage((Message) p);
				}
			}, new PacketTypeFilter(Message.class));
	
			// Listens to presence messages
			con.addPacketListener(new PacketListener()
			{
				public void processPacket(Packet p)
				{
					handlePresence((Presence) p);
				}
			}, new PacketTypeFilter(Presence.class));
	
			// Workaround: Do not continue until the roster is recieved from the
			// server
			while (con.getRoster() == null)
				Thread.sleep(50);
	
			this.RosterChanged.fire("");
	
			// Listens to roster and presence changes
			con.getRoster().addRosterListener(new RosterListener()
			{
				public synchronized void rosterModified()
				{
					RosterChanged.fire("");
				}
	
				public synchronized void presenceChanged(String strUser)
				{
					onPresenceChanged(strUser);
				}
			});
			this.Connected.fire("");
		}
		catch (Exception e)
		{
			GuiUtil.handleException(e,"Connection attempt failed");
		}
	}

	public void closeConnection()
	{
		if (this.con != null) this.con.close();
		this.con = null;
		this.Disconnected.fire("");
	}

	public XMPPConnection getConnection()
	{
		return con;
	}
	
	public boolean isConnected()
	{
		return (this.con != null);
	}

	public static SimpleChatPlugin getDefault()
	{
		return plugin;
	}
	
	public void onFileFocusChanged(IEditorPart part)
	{
		// Update own decorators
		IEditorPart edtPart = (IEditorPart) part;
		if(edtPart.getEditorInput() instanceof IFileEditorInput)
		{
			IFileEditorInput fedInp = (IFileEditorInput) edtPart.getEditorInput();
			final String strFile = fedInp.getFile().getFullPath()
					.toPortableString();
			final String strMe = "$me";
			final String strProperty = "FileFocus";
	
			try
			{
				AwarenessEntry awEntry = logAwareness.addIncomingAwarenessMsg(
						strMe, strFile, Util.getDateFormatter().format(new Date()),
						strProperty, "", "");
				htLatestAwareness.put(awEntry.getWho(), awEntry);
				Display.getDefault().syncExec(new Runnable()
				{
					public void run()
					{
						updateDecorators();
					}
				});
	
				Thread.sleep(100); // HACK: Fixes a rare problem with decorator
				// updating
	
				
				// Send awareness information
				XMPPConnection con = this.getConnection();
				if ((con != null) && (con.isConnected()))
				{
					Roster r = con.getRoster();
					Project prj = ProjectConfig.getConfig().resolveProjectFromIProject(fedInp.getFile().getProject());
					if(prj != null)
					{
						Collection<Contact> collMembs = prj.getMembers();
						for (Contact contact : collMembs)
						{
							String strUser = contact.getUserName();
							boolean bOnline = (r.getPresence(strUser) != null);
							if(bOnline)
							{
								Message m = new Message(strUser, Message.Type.CHAT);
								m.setBody(AWARENESS_MSG_ID);
								m.setProperty(strProperty, strFile);
								con.sendPacket(m);
							}
						}
					}
				}
	
			}
			catch (Exception ex)
			{
				GuiUtil.handleException(ex);
			}
		}
	}
	
	private void handleAwarenessMessage(Message m)
	{
		String strProperty = (String) (m.getPropertyNames().next());
		String strFile = (String) m.getProperty(strProperty);
		String strUser = m.getFrom();
		try
		{
			AwarenessEntry awEntry = logAwareness.addIncomingAwarenessMsg(
					strUser, strFile, Util.getDateFormatter()
							.format(new Date()), strProperty, "", "");
			htLatestAwareness.put(awEntry.getWho(), awEntry);
			Display.getDefault().syncExec(new Runnable()
			{
				public void run()
				{
					updateDecorators();
				}
			});
		}
		catch (Exception ex)
		{
			GuiUtil.handleException(ex);
		}
	}
	
	private void handleMessage(Message m)
	{
		final String strBody = m.getBody();

		// Check if it is an awareness message
		if (strBody.equals(SimpleChatPlugin.AWARENESS_MSG_ID))
		{
			handleAwarenessMessage(m);
			return;
		}

		String strContextTmp = (String) m.getProperty(SimpleChatPlugin.CONTEXT_PROPERTY_ID);
		final String strContext;
		if(Util.isStringEmpty(strContextTmp, LogSession.DEFAULT_CONTEXT_NAME)) strContext = LogSession.DEFAULT_CONTEXT_NAME;
		else strContext = strContextTmp;
		
		String strProjectTmp = (String) m.getProperty(SimpleChatPlugin.PROJECT_PROPERTY_ID);
		final String strProject;
		if (Util.isStringEmpty(strProjectTmp, LogSession.DEFAULT_PROJECT_NAME)) strProject = LogSession.DEFAULT_PROJECT_NAME;
		else strProject = strProjectTmp;
		
		String strLocationTmp = (String) m.getProperty(SimpleChatPlugin.CONTEXT_LOCATION_PROPERTY_ID);
		final String strLocation;
		if(Util.isStringEmpty(strLocationTmp, LogSession.DEFAULT_CONTEXT_LOCATION)) strLocation = LogSession.DEFAULT_CONTEXT_LOCATION;
		else strLocation = strLocationTmp;
		
		String strMoreInfoTmp = (String) m.getProperty(SimpleChatPlugin.MORE_INFO_PROPERTY_ID);
		final String strMoreInfo;
		if(strMoreInfoTmp == null) strMoreInfo = "";
		else strMoreInfo = strMoreInfoTmp;
		
		final String strFrom = Util.stripUserString(m.getFrom());
		LogData ld = this.htLogDataObjs.get(strFrom);
		if(ld == null) 
		{
			try
			{
				ld = LogData.create(strFrom);
			}
			catch (IOException ex)
			{
				// TODO Auto-generated catch block
				ex.printStackTrace();
			}
		}
		final LogData cs = ld;
		Display.getDefault().syncExec(new Runnable()
		{
			public void run()
			{
				// Open chat GUI
				ActivateChatView.fire(new ActivateViewData(cs, cs.getContactNode()));
				try
				{
					cs.handleMessage(strFrom, strBody, strContext, strProject, strLocation, strMoreInfo);
				}
				catch (UnsupportedEncodingException ex)
				{
					// TODO Auto-generated catch block
					ex.printStackTrace();
				}
			}
		});
	}
	
	private void updateDecorators()
	{
		this.getWorkbench().getDecoratorManager()
				.update("no.jabber.eclipseplugin.decorators.filedecorator");
	}
	
	public AwarenessEntry[] getLatestAwareness()
	{
		AwarenessEntry[] aw = new AwarenessEntry[htLatestAwareness.size()];
		return htLatestAwareness.values().toArray(aw);
	}
	
	/**
	 * Called when there is a presence-chance in the roster Updates the display
	 * of the roster
	 * 
	 * @param strUser
	 *            The user's whose presence info changed
	 */
	private void onPresenceChanged(String strUser)
	{
		final Roster r = this.getConnection().getRoster();
		final boolean bOnline = (r.getPresence(strUser) != null);
		final String strUserStripped = Util.stripUserString(strUser);

		Display.getDefault().syncExec(new Runnable()
		{
			public void run()
			{
				firePresenceChanged(bOnline,strUserStripped);
			}
		});
	}

	protected void firePresenceChanged(boolean bOnline,String strUser)
	{
		PresenceChanged.fire(new PresenceChangedData(bOnline,strUser));
	}
	
	private void handlePresence(final Presence p)
	{
		Presence.Type type = p.getType();
		final String strUser = p.getFrom();
		final Roster r = this.getConnection().getRoster();

		if (type == Presence.Type.SUBSCRIBE)
		{
			// Someone wants to add us their roster (this request is
			// automatically accepted by the framework).
			if (r.getEntry(strUser) != null) return;

			// If we do not have this user in our roster, open a dialog to add
			// this user
			Display.getDefault().syncExec(new Runnable()
			{
				public void run()
				{
					NewContactOnRoster.fire(new NewContactOnRosterData(strUser,r));
				}
			});
		}
		else if (type == Presence.Type.UNSUBSCRIBED)
		{
			// Someone has removed us from their roster
			try
			{
				// We remove them from our roster too
				r.removeEntry(r.getEntry(strUser));
			}
			catch (XMPPException e)
			{
				GuiUtil.handleException(e);
			}
		}

	}
	
	public Collection<Contact> getAllContactsFromRoster()
	{
		Roster r = this.getConnection().getRoster();

		@SuppressWarnings("unchecked")
		Iterator<RosterEntry> i = r.getEntries();
		final ArrayList<Contact> alRosterItems = new ArrayList<Contact>();
		while (i.hasNext())
		{
			RosterEntry e = i.next();
			String strUser = e.getUser();
			Presence p = r.getPresence(strUser);
			Contact cont = new Contact(strUser, e.getName(), (p != null));
			if (cont.getFriendlyName() == null) cont.setFriendlyName(cont
					.getUserName());
			alRosterItems.add(cont);
		}
		return alRosterItems;
	}

	public void setActiveSession(LogSession session)
	{
		this.sessionActive = session;
	}

	public void setContextLocationFromMenu(EclipseLink eclLink)
	{
		if((sessionActive != null) && (eclLink != null)) sessionActive.setContextLocation(eclLink.getLinkAsString()); 
	}
}