package no.jabber.eclipseplugin;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import no.jabber.eclipseplugin.SimpleChatView.NodeClass;
import no.jabber.eclipseplugin.Util.CustomEvent;
import no.jabber.eclipseplugin.Util.CustomListener;
import no.jabber.eclipseplugin.dialogs.ConfigProjectDialog;
import no.jabber.eclipseplugin.dialogs.ErrorDialogs;
import no.jabber.eclipseplugin.dialogs.RosterEntryDialog;
import no.jabber.eclipseplugin.projectmanagement.Contact;
import no.jabber.eclipseplugin.projectmanagement.Project;
import no.jabber.eclipseplugin.projectmanagement.ProjectConfig;
import no.jabber.eclipseplugin.resources.ImageCollection;
import no.jabber.eclipseplugin.serialization.LogData;
import no.jabber.eclipseplugin.serialization.LogEntry;
import no.jabber.eclipseplugin.serialization.LogSession;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.part.ViewPart;
import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.XMPPException;

public class SimpleChatRoster extends ViewPart
{
	private FormToolkit toolkit;
	private Hyperlink btnConnect;
	private Tree tree;
	private TreeItem nodeRoster;
	private Menu menuContactNode;

	public static SimpleChatRoster Instance = null;
	private Composite pnlLeft;
	private Hashtable<CustomEvent,Collection<CustomListener>> htSessionEvents = new Hashtable<CustomEvent, Collection<CustomListener>>();
	private CustomListener<NewContactOnRosterData> ltrNewContactOnRoster;
	private CustomListener<String> ltrRosterModified;
	private CustomListener<String> ltrDisconnected;
	private CustomListener<String> ltrConnected;
	private CustomListener<PresenceChangedData> ltrPresence;
	
	public SimpleChatRoster()
	{
		super();
		Instance = this;
	}

	@Override
	public void createPartControl(Composite parent)
	{
		this.setPartName("Chat Roster");
		this.initForm(parent);
//		SimpleChatPlugin.getDefault().setRoster(this);
		this.hookEvents();
	}

	private void hookEvents()
	{
		ltrPresence = new CustomListener<PresenceChangedData>(){
					public void onAction(PresenceChangedData data)
					{
						// Update the roster
						Iterator<TreeItem> i = Arrays.asList(nodeRoster.getItems())
								.iterator();
						while (i.hasNext())
						{
							TreeItem item = i.next();
							String strUserInTable = (String) item.getData(GuiItemsDataKeys.TREEITEM_CONTACT_AS_STRING);
							if (data.getUser().equals(strUserInTable))
							{
								item.setImage(ImageCollection.getInstance().get(data.isOnline() ? "online" : "offline"));
								return;
							}
						}
					}};
		SimpleChatPlugin.getDefault().PresenceChanged.addListener(ltrPresence);
		
		ltrConnected = new CustomListener<String>(){
					public void onAction(String data)
					{
						btnConnect.setText("Disconnect");
						btnConnect.setToolTipText("Disconnect from server");
						btnConnect.setData(SimpleChatPlugin.BTNCONNECT_DISCONNECT);
						ProjectConfig.getConfig().ConfigurationUpdated.fire(ProjectConfig.getConfig().getProjects());
					}};
		SimpleChatPlugin.getDefault().Connected.addListener(ltrConnected);
		
		ltrDisconnected = new CustomListener<String>(){
					@SuppressWarnings("unchecked")
					public void onAction(String data)
					{
						if(!btnConnect.isDisposed())
						{
							btnConnect.setText("Connect");
							btnConnect.setToolTipText("Connect to server");
							btnConnect.setData(SimpleChatPlugin.BTNCONNECT_CONNECT);
						}
						
						// Remove listeners on session objects
						Enumeration<CustomEvent> events = htSessionEvents.keys();
						while(events.hasMoreElements())
						{
							CustomEvent event = events.nextElement();
							Collection<CustomListener> listeners = htSessionEvents.get(event);
							for (Iterator iter = listeners.iterator(); iter.hasNext();)
							{
								CustomListener listener = (CustomListener) iter.next();
								event.removeListener(listener);				
							}
						}
						htSessionEvents.clear();
						if(!nodeRoster.isDisposed())
						{
							nodeRoster.removeAll();
						}
					}};
		SimpleChatPlugin.getDefault().Disconnected.addListener(ltrDisconnected);
		
		ltrRosterModified = new CustomListener<String>(){
					public void onAction(String data)
					{
						onRosterModified();
					}};
		SimpleChatPlugin.getDefault().RosterChanged.addListener(ltrRosterModified);
		
		ltrNewContactOnRoster = new CustomListener<NewContactOnRosterData>(){
					public void onAction(NewContactOnRosterData data)
					{
						Shell shell = getViewSite().getShell();
						RosterEntryDialog dlg = new RosterEntryDialog(shell);
						dlg.setUser(data.getUser(), true);
						if (dlg.open() == Window.OK)
						{
							String strName = dlg.getName();
							try
							{
								data.getRoster().createEntry(data.getUser(), strName, null);
							}
							catch (XMPPException e)
							{
								GuiUtil.handleException(e);
							}
						}
					}};
		SimpleChatPlugin.getDefault().NewContactOnRoster.addListener(ltrNewContactOnRoster);
		
		ProjectConfig.getConfig().ConfigurationUpdated.addListener(new CustomListener<Collection<Project>>(){
			public void onAction(Collection<Project> data)
			{
				for(Project prj : data)
				{
					Hashtable<String,Contact> htMembers = prj.getMembersInHt();
					for(TreeItem nodeContact : nodeRoster.getItems())
					{
						if(htMembers.get(nodeContact.getText()) != null)
						{
							boolean bNodeExists = false;
							for(TreeItem nodePrjs : nodeContact.getItems())
							{
								if(nodePrjs.getText().equalsIgnoreCase(prj.getCommonName()))
								{
									bNodeExists = true;
									break;
								}
							}
							if(!bNodeExists) makeProjectNode(nodeContact, prj.getCommonName());
						}
					}
				}
			}});
	}

	@Override
	public void dispose()
	{
		SimpleChatPlugin.getDefault().closeConnection();
		this.unhookEvents();
		super.dispose();
	}

	private void unhookEvents()
	{
		if(ltrConnected != null) SimpleChatPlugin.getDefault().Connected.removeListener(ltrConnected);
		if(ltrDisconnected != null) SimpleChatPlugin.getDefault().Disconnected.removeListener(ltrDisconnected);
		if(ltrNewContactOnRoster != null) SimpleChatPlugin.getDefault().NewContactOnRoster.removeListener(ltrNewContactOnRoster);
		if(ltrPresence != null) SimpleChatPlugin.getDefault().PresenceChanged.removeListener(ltrPresence);
		if(ltrRosterModified != null) SimpleChatPlugin.getDefault().RosterChanged.removeListener(ltrRosterModified);
	}

	private void initForm(Composite parent)
	{
		parent.setLayout(new GridLayout(1, true));
		toolkit = new FormToolkit(parent.getDisplay());
		toolkit.adapt(parent);
		final Form frmEclipse = toolkit.createForm(parent);
		frmEclipse.setLayoutData(GuiUtil.createFillGridData());
		GridLayout grdClient = new GridLayout(1, true);
		frmEclipse.getBody().setLayout(grdClient);

		pnlLeft = toolkit.createComposite(frmEclipse.getBody());
		GridData grdLeft = new GridData();
		grdLeft.grabExcessVerticalSpace = true;
		grdLeft.verticalAlignment = SWT.FILL;
		grdLeft.horizontalAlignment = SWT.LEFT;
		pnlLeft.setLayout(new GridLayout());
		pnlLeft.setLayoutData(GuiUtil.createFillGridData());
		
		final Section secToolbar = toolkit.createSection(pnlLeft,
				Section.TITLE_BAR | Section.TWISTIE);
		secToolbar.setText("Actions");
		GridData grdToolbar = new GridData();
		grdToolbar.grabExcessHorizontalSpace = true;
		grdToolbar.horizontalAlignment = SWT.FILL;
		secToolbar.setLayoutData(grdToolbar);
		Composite pnlToolbar = toolkit.createComposite(secToolbar);
		pnlToolbar.setLayout(new GridLayout(2,false));
		btnConnect = toolkit.createHyperlink(pnlToolbar, "Connect", SWT.NONE);
		btnConnect.setToolTipText("Connect to server");
		btnConnect.setData(SimpleChatPlugin.BTNCONNECT_CONNECT);
		btnConnect.addHyperlinkListener(new HyperlinkAdapter()
		{
			public void linkActivated(HyperlinkEvent e)
			{
				onBtnConnect_Click(false);
			}
		});
		Hyperlink hlAddContact = toolkit.createHyperlink(pnlToolbar, "Add contact", SWT.NONE);
		hlAddContact.setToolTipText("Add a contact to roster");
		hlAddContact.addHyperlinkListener(new HyperlinkAdapter()
		{
			public void linkActivated(HyperlinkEvent e)
			{
				onAddContactMenu_Click();
			}
		});
			
		secToolbar.setClient(pnlToolbar);
		
		tree = toolkit.createTree(pnlLeft, SWT.SINGLE);
		tree.setLayoutData(GuiUtil.createFillGridData());
		tree.addKeyListener(new KeyAdapter(){
			public void keyPressed(KeyEvent e)
			{
				if(e.keyCode == SWT.F2)
				{
					if(tree.getSelectionCount() > 0)
					{
						TreeItem it = tree.getSelection()[0];
						NodeClass nc = (NodeClass) it.getData(GuiItemsDataKeys.TREEITEM_NODECLASS);
						if(nc != null)
						{
							if(nc == NodeClass.Conversation) renameContextInTree();
						}
					}
				}
			}
			});
		
		nodeRoster = new TreeItem(tree, SWT.NONE);
		nodeRoster.setText("Contacts");
		nodeRoster.setImage(ImageCollection.getInstance().get("icoRoster"));
		
		tree.addMouseListener(new MouseAdapter()
		{
			public void mouseDoubleClick(MouseEvent e)
			{
				TreeItem item = tree.getItem(new Point(e.x, e.y));
				if (item != null)
				{
					if(item.getData(GuiItemsDataKeys.TREEITEM_ROSTERNODE) != null) 
					{
						onRosterItemActivated((TreeItem)item.getData(GuiItemsDataKeys.TREEITEM_ROSTERNODE));
						return;
					}
					if (item.getData(GuiItemsDataKeys.TREEITEM_NODECLASS) != null)
					{
						NodeClass cl = (NodeClass) item.getData(GuiItemsDataKeys.TREEITEM_NODECLASS);
						if (cl == NodeClass.Conversation) 
						{
							onConversationNodeDblClick(item);
							return;
						}
						if (cl == NodeClass.Contact)
						{
							onRosterItemActivated(item);
							return;
						}
					}
				}
			}
		});
		
		this.createRosterPopupMenu();
	}

	private void createRosterPopupMenu()
	{
//		 TODO: Lag sepparatorer snn at menyen blir nice og forstelig
		// TODO: Lag ikoner for menyvalgene
		final Action acRemoveContact = new Action()
		{
			// "Remove Contact"
			public void run()
			{
				onRemoveContactMenu_Click();
			}
		};

		final Action acAddContact = new Action()
		{
			// "Add Contact..."
			public void run()
			{
				onAddContactMenu_Click();
			}
		};
		
		final Action acConfigureProjects = new Action()
		{
			public void run()
			{
				openProjectConfigDialog();
			}
		};
		
		// Only availble when clicking a project
		final Action acNewContext = new Action()
		{
			public void run()
			{
				TreeItem nodeProj = tree.getSelection()[0];
				TreeItem nodeContact = nodeProj.getParentItem();
				LogData ld = (LogData) nodeContact.getData(GuiItemsDataKeys.TREEITEM_LOGDATA);
				ld.createNewSession("new context", "", nodeProj.getText());
				TreeItem nodeNewCtx = nodeProj.getItems()[nodeProj.getItemCount()-1];
				tree.setSelection(new TreeItem[]{nodeNewCtx});
				renameContextInTree();
			}
		};
		
		final Action acDeleteContext = new Action()
		{
			public void run()
			{
				// TODO: gjr deill
			}
		};
		
		final Action acConnect = new Action()
		{
			public void run()
			{
				onBtnConnect_Click(true);
			}
		};
		
		final Action acDisconnect = new Action()
		{
			public void run()
			{
				onBtnConnect_Click(true);
			}
		};
		
		final Action acRenameContext = new Action()
		{
			public void run()
			{
				renameContextInTree();
			}
		};
		
		
		acRemoveContact.setText("Remove contact");
		acAddContact.setText("Add contact...");
		acConfigureProjects.setText("Configure projects..");
		acNewContext.setText("New context");
		acDeleteContext.setText("Delete context");
		acDeleteContext.setEnabled(false);
		acConnect.setText("Connect");
		acDisconnect.setText("Disconnect");
		acRenameContext.setText("Rename context");
		
		//acRenameContext.setAccelerator(SWT.F2);
		
		MenuManager mm = new MenuManager("#ContactsPopup");
		mm.setRemoveAllWhenShown(true);
		mm.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager mm)
			{
				boolean bConnected = SimpleChatPlugin.getDefault().isConnected();
				if(bConnected) mm.add(acDisconnect);
				else mm.add(acConnect);
				// Separator her...
				mm.add(new Separator());
				mm.add(acAddContact);
				acAddContact.setEnabled(bConnected);
				if(tree.getSelectionCount() > 0)
				{
					final TreeItem nodeSelected = tree.getSelection()[0];
					NodeClass nc = (NodeClass) nodeSelected.getData(GuiItemsDataKeys.TREEITEM_NODECLASS);
					if(nc != null)
					{
						if(nc == NodeClass.Contact)
						{
							// only possible to remove contacts when a contact is clicked
							mm.add(acRemoveContact);
						}
						else if (nc == NodeClass.Conversation)
						{
							mm.add(new Separator());
							mm.add(acDeleteContext);
							mm.add(acRenameContext);
							MenuManager mmSetProject = new MenuManager("Set project","#SetProject");
							for(final Project p : ProjectConfig.getConfig().getProjects())
							{
								final Action ac = new Action()
								{
									public void run()
									{
										LogSession ls = (LogSession)nodeSelected.getData(GuiItemsDataKeys.TREEITEM_LOGSESSION);
										ls.setProject(p.getCommonName());
									}
								};
								ac.setText(p.getCommonName());
								mmSetProject.add(ac);
								ac.setEnabled(false);
							}
							mm.add(mmSetProject);
						}
						else if (nc == NodeClass.Project)
						{
							mm.add(new Separator());
							mm.add(acNewContext);
						}
					}
				}
				mm.add(new Separator());
				mm.add(acConfigureProjects);
				acConfigureProjects.setEnabled(bConnected);
			}
		});

		menuContactNode = mm.createContextMenu(this.tree);
		this.tree.setMenu(menuContactNode);
	}
	
	protected void openProjectConfigDialog()
	{
//			if(SimpleChatPlugin.getDefault().isConnected())
//			{
//				// Open configuration dialog
//				if (tree.getSelection()[0].equals(nodeConfigureProjects))
//				{
					ConfigProjectDialog dlg = new ConfigProjectDialog(GuiUtil
							.getCurrentShell(), SimpleChatPlugin.getDefault().getAllContactsFromRoster(), GuiUtil
							.getCurrentWorkspace().getRoot().getProjects());
					if (dlg.open())
					{
						ProjectConfig.getConfig().saveConfig();
					}
//				}
//			}
//			else ErrorDialogs.openNotConnectedError(tree.getShell());
	}

	protected void onRemoveContactMenu_Click()
	{
		TreeItem nodeCurrent = this.tree.getSelection()[0];
		if(nodeCurrent.getData(GuiItemsDataKeys.TREEITEM_NODECLASS) == NodeClass.Contact)
		{
			String strUser = (String) nodeCurrent.getData(GuiItemsDataKeys.TREEITEM_CONTACT_AS_STRING);
			if(MessageDialog.openQuestion(nodeCurrent.getParent().getShell(), "Confirm remove user", "Are you sure you want to remove user "+strUser+" from the roster?"))
			{
			Roster r = SimpleChatPlugin.getDefault().getConnection().getRoster();
			RosterEntry re = r.getEntry(strUser);
				try
				{
					LogData ld = SimpleChatPlugin.getDefault().htLogDataObjs.get(strUser);
					ld.dispose(true);
					SimpleChatPlugin.getDefault().htLogDataObjs.remove(strUser);
					r.removeEntry(re);
					nodeCurrent.removeAll();
					nodeCurrent.dispose();
				}
				catch (XMPPException e)
				{
					GuiUtil.handleException(e);
				}
			}
		}
	}

	protected void onConversationNodeDblClick(TreeItem item)
	{
		LogSession session = (LogSession) item.getData(GuiItemsDataKeys.TREEITEM_LOGSESSION);
		TreeItem nodeContact = item.getParentItem().getParentItem();
		SimpleChatPlugin.getDefault().ActivateChatView.fire(new ActivateViewData(session.getParent(),nodeContact));
		session.setActive(true);
	}

	protected void onRosterItemActivated(TreeItem item)
	{
		if(item.getItemCount() > 0)
		{
			TreeItem it = item.getItem(0);
			if(it.getItemCount() > 0)
			{
				it = it.getItem(0);
				tree.showItem(it);
				LogSession s = (LogSession)it.getData(GuiItemsDataKeys.TREEITEM_LOGSESSION);
				SimpleChatPlugin.getDefault().ActivateChatView.fire(new ActivateViewData(s.getParent(),item));
				s.setActive(true);
			}
		}
	}

	protected void renameContextInTree()
	{
	
		final TreeEditor te = new TreeEditor(tree);
		// Dispose any old editors
		Control oldEditor = te.getEditor();
		if (oldEditor != null) oldEditor.dispose();
		
		final TreeItem it = tree.getSelection()[0];
		final Text txt = new Text(tree,SWT.NONE);
		toolkit.adapt(txt, true, true);
		te.horizontalAlignment = SWT.LEFT;
		te.grabHorizontal = false;
		te.minimumWidth = it.getBounds().width + 2;
		te.minimumHeight = it.getBounds().height;
		te.setEditor (txt, it);
		toolkit.paintBordersFor(tree);
		txt.setText(it.getText());
		txt.selectAll();
		txt.setCursor(new Cursor(txt.getDisplay(),SWT.CURSOR_IBEAM));
		txt.setFocus();
		txt.addKeyListener(new KeyListener(){

			public void keyPressed(KeyEvent e)
			{
				
			}

			public void keyReleased(KeyEvent e)
			{
				if(e.keyCode == SWT.ESC)
				{
					txt.dispose();
					tree.redraw();
					return;
				}
				if(e.keyCode == SWT.CR)
				{
					LogSession s = (LogSession) it.getData(GuiItemsDataKeys.TREEITEM_LOGSESSION);
					s.setContext(txt.getText());
					txt.dispose();
					tree.redraw();
				}
			}});
		txt.addFocusListener(new FocusListener(){
			public void focusGained(FocusEvent e)
			{
				
			}

			public void focusLost(FocusEvent e)
			{
				LogSession s = (LogSession) it.getData(GuiItemsDataKeys.TREEITEM_LOGSESSION);
				s.setContext(txt.getText());
				txt.dispose();
				tree.redraw();
			}});
		tree.redraw();
	
	}

	protected void onAddContactMenu_Click()
	{
		if(SimpleChatPlugin.getDefault().isConnected())
		{
			RosterEntryDialog dlg = new RosterEntryDialog(tree.getShell());
			if (dlg.open() == Window.OK)
			{
				String strUser = dlg.getUser();
				String strName = dlg.getName();
				Roster r = SimpleChatPlugin.getDefault().getConnection()
						.getRoster();
				try
				{
					r.createEntry(strUser, strName, null);
				}
				catch (XMPPException e)
				{
					GuiUtil.handleException(e);
				}
			}
		} else ErrorDialogs.openNotConnectedError(this.tree.getShell());
	}

	
	
	/**
	 * Called when the user clicks the Connect/Disconnect-button Inits or closes
	 * the connection
	 */
	protected void onBtnConnect_Click(boolean bCalledFromMenu)
	{
		pnlLeft.getParent().setCursor(new Cursor(pnlLeft.getDisplay(),SWT.CURSOR_WAIT));
		String str = (String) btnConnect.getData();

		if (str.equals(SimpleChatPlugin.BTNCONNECT_CONNECT))
		{
			try
			{
				SimpleChatPlugin.getDefault().initConnection();
			}
			catch (XMPPException ex)
			{
				// TODO Auto-generated catch block
				ex.printStackTrace();
			}
		}
		else if (str.equals(SimpleChatPlugin.BTNCONNECT_DISCONNECT))
		{
			SimpleChatPlugin.getDefault().closeConnection();
		}
		if(!bCalledFromMenu) btnConnect.getParent().pack();
		pnlLeft.getParent().setCursor(new Cursor(pnlLeft.getDisplay(),SWT.CURSOR_ARROW));
	}
	
	private void onRosterModified()
	{
		Display.getDefault().syncExec(new Runnable()
		{
			public void run()
			{
				updateRoster(SimpleChatPlugin.getDefault().getAllContactsFromRoster());
			}
		});
	}

	private void updateRoster(Collection<Contact> collRosterItems)
	{
		for (Contact contact : collRosterItems)
		{
			LogData ld = SimpleChatPlugin.getDefault().htLogDataObjs.get(contact.getUserName());
			if (ld == null)
			{
				try
				{
					ld = LogData.create(contact.getUserName());
					SimpleChatPlugin.getDefault().htLogDataObjs.put(contact.getUserName(), ld);
				}
				catch (Exception ex)
				{
					GuiUtil.handleException(ex);
				}
			}
			contact.setLogData(ld);
			if(ld.getContactNode() == null)
			{
				final TreeItem nodeContact = this.makeContactNode(nodeRoster, contact);
				nodeContact.setData(GuiItemsDataKeys.TREEITEM_LOGDATA,ld);
				ld.setContactNode(nodeContact);
				if (ld.SessionAdded == null) ld.SessionAdded = new CustomEvent<LogSession>();
				ld.SessionAdded.addListener(new CustomListener<LogSession>()
				{
					public void onAction(LogSession s)
					{
						makeConversationNode(nodeContact, s);
					}
				});
				for (LogSession s : ld.getSessions())
				{
					this.makeConversationNode(nodeContact, s);
				}
			}
		}
		nodeRoster.setExpanded(true);
	}

	
	private TreeItem makeConversationNode(TreeItem nodeContact, final LogSession session)
	{
		String strPrj = session.getProject();
		if( Util.isStringEmpty(strPrj, LogSession.DEFAULT_PROJECT_NAME)) strPrj = "(none)";
		
		// Find correct project node under the contact
		TreeItem nodePrj = null;
		for(TreeItem n : nodeContact.getItems())
		{
			if(n.getText().equalsIgnoreCase(strPrj))
			{
				nodePrj = n;
				break;
			}
		}
		if(nodePrj == null)
		{
			nodePrj = this.makeProjectNode(nodeContact,strPrj);
		}
		final TreeItem node = new TreeItem(nodePrj, SWT.NONE);
		node.setText(session.getContext());
		node.setImage(ImageCollection.getInstance().get("icoConv"));
		node.setData(GuiItemsDataKeys.TREEITEM_NODECLASS, NodeClass.Conversation);
		node.setData(GuiItemsDataKeys.TREEITEM_LOGSESSION, session);
		if (session.ContextChanged == null) session.ContextChanged = new CustomEvent<String>();
		if (session.ProjectChanged == null) session.ProjectChanged = new CustomEvent<String>();
		if (session.ActiveStateChanged == null) session.ActiveStateChanged = new CustomEvent<Boolean>();
		if (session.MessageAdded == null) session.MessageAdded = new CustomEvent<LogEntry>();
		
		CustomListener<String> ltrContext = new CustomListener<String>()
		{
			public void onAction(String data)
			{
				node.setText(session.getContext());
			}
		};
		CustomListener<String> ltrPrj = new CustomListener<String>()
		{
			public void onAction(String data)
			{
				// dispose the node
				// recreate it on the correct project node
				TreeItem nodeContact = node.getParentItem().getParentItem();
				//node.dispose();
				String strPrj = session.getProject();
				if(Util.isStringEmpty(strPrj, LogSession.DEFAULT_PROJECT_NAME)) strPrj = "(none)";

				// Find correct project node under the contact
				TreeItem nodePrj = null;
				for(TreeItem n : nodeContact.getItems())
				{
					if(n.getText().equalsIgnoreCase(strPrj))
					{
						nodePrj = n;
						break;
					}
				}
				if(nodePrj == null)
				{
					nodePrj = makeProjectNode(nodeContact,strPrj);
				}
				//makeConversationNode(nodePrj, session);
				// TODO: Noden m flyttes til en annen node prosjektnode...
//				node.setText(session.getProject() + "," + session.getContext());
			}
		};
		CustomListener<Boolean> ltrAct = new CustomListener<Boolean>()
		{
			public void onAction(Boolean data)
			{
				if(data)
				{
					node.setImage(ImageCollection.getInstance().get("icoConvActive"));
					// Un-boldify font
					// --- conv.node
					node.setFont(GuiUtil.boldifyFont(false, node.getFont()));
					// --- project node
					node.getParentItem().setFont(GuiUtil.boldifyFont(false, node.getParentItem().getFont()));
					// -- contact node
					node.getParentItem().getParentItem().setFont(GuiUtil.boldifyFont(false, node.getParentItem().getParentItem().getFont()));
//					TODO: Fix!!
//					updateInputControlsForPresence(session.getParent().getYouUsername());
					SimpleChatPlugin.getDefault().setActiveSession(session);
				}
				else
				{
					if(!node.isDisposed()) node.setImage(ImageCollection.getInstance().get("icoConv"));
				}
			}
		};
		CustomListener<LogEntry> ltrAdded = new CustomListener<LogEntry>()
		{
			public void onAction(LogEntry data)
			{
				if(!session.isActive())
				{
//					Boldify font
					// --- conv.node
					node.setFont(GuiUtil.boldifyFont(true, node.getFont()));
//					 --- project node
					node.getParentItem().setFont(GuiUtil.boldifyFont(true, node.getParentItem().getFont()));
//					 -- contact node
					node.getParentItem().getParentItem().setFont(GuiUtil.boldifyFont(true, node.getParentItem().getParentItem().getFont()));
				}
			}
		};
		
		session.ContextChanged.addListener(ltrContext);
		session.ProjectChanged.addListener(ltrPrj);
		session.ActiveStateChanged.addListener(ltrAct);
		session.MessageAdded.addListener(ltrAdded);
		
		ArrayList<CustomListener> alCtxListeners = new ArrayList<CustomListener>();
		alCtxListeners.add(ltrContext);
		htSessionEvents.put(session.ContextChanged, alCtxListeners);
		
		ArrayList<CustomListener> alPrjListeners = new ArrayList<CustomListener>();
		alPrjListeners.add(ltrPrj);
		htSessionEvents.put(session.ProjectChanged, alPrjListeners);
		
		ArrayList<CustomListener> alActListeners = new ArrayList<CustomListener>();
		alActListeners.add(ltrAct);
		htSessionEvents.put(session.ActiveStateChanged, alActListeners);
		
		ArrayList<CustomListener> alMsgListeners = new ArrayList<CustomListener>();
		alMsgListeners.add(ltrAdded);
		htSessionEvents.put(session.MessageAdded, alMsgListeners);
		
		return node;
	}
	
	private TreeItem makeContactNode(TreeItem nodeParent, Contact cont)
	{
		TreeItem item = new TreeItem(nodeParent, SWT.NONE);
		item.setData(GuiItemsDataKeys.TREEITEM_NODECLASS, NodeClass.Contact);
		item.setImage(ImageCollection.getInstance().get(
				cont.isOnline() ? "online" : "offline"));
		item.setData(GuiItemsDataKeys.TREEITEM_CONTACT_AS_STRING, cont.getUserName());
		item.setText(cont.getFriendlyName());
		return item;
	}
	
	private TreeItem makeProjectNode(TreeItem nodeParent, String strPrj)
	{
		TreeItem it = new TreeItem(nodeParent,SWT.NONE);
		it.setData(GuiItemsDataKeys.TREEITEM_NODECLASS, NodeClass.Project);
		it.setImage(ImageCollection.getInstance().get("icoProject"));
		it.setText(strPrj);
		return it;
	}
	
	@Override
	public void setFocus(){}

}
