package no.jabber.eclipseplugin;

import no.jabber.eclipseplugin.linking.EclipseLink;
import no.jabber.eclipseplugin.serialization.LogData;
import no.jabber.eclipseplugin.serialization.LogSession;
import no.jabber.eclipseplugin.Util.CustomEvent;
import no.jabber.eclipseplugin.Util.CustomListener;
import no.jabber.eclipseplugin.dialogs.*;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.part.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.events.*;
import org.jivesoftware.smack.packet.*;

/**
 * This singleton class is the main GUI class of the plug-in.
 */
public class SimpleChatView extends ViewPart
{
	public static String INPUT_DISABLED = "input_disabled";
	public static String INPUT_ENABLED = "input_enabled";

	private Composite pnlRight;
	private CTabFolder tfFreeChats;
	private Text txtInput;
	private Button btnSend;
	private FormToolkit toolkit;

	private IPartListener ltrAwareness;
	private CustomListener<ActivateViewData> ltrActivateChatView;
	private CustomListener<PresenceChangedData> ltrPresenceChanged;
	private CustomListener<String> ltrDisconnected;
	private CustomListener<EclipseLink> ltrEclLink;

	public SimpleChatView(){}

	public void createPartControl(Composite parent)
	{
		this.setPartName("Chat");
		this.initForm(parent);
		SimpleChatPlugin.getDefault().setView(this);
		this.hookEvents();
	}
	
	private void hookEvents()
	{
		ltrAwareness = new IPartListener()
		{
			public void partActivated(IWorkbenchPart part)
			{
				if (part instanceof IEditorPart)
				{
					SimpleChatPlugin.getDefault().onFileFocusChanged((IEditorPart) part);
				}
			}

			public void partBroughtToTop(IWorkbenchPart part)
			{
			}

			public void partClosed(IWorkbenchPart part)
			{
			}

			public void partDeactivated(IWorkbenchPart part)
			{
			}

			public void partOpened(IWorkbenchPart part)
			{
			}
		};
		this.getSite().getPage().addPartListener(ltrAwareness);
		
		
		ltrActivateChatView = new CustomListener<ActivateViewData>(){
					public void onAction(ActivateViewData data)
					{
						createChatView(data.getLogData(), data.getContactNode());
					}};
		SimpleChatPlugin.getDefault().ActivateChatView.addListener(ltrActivateChatView);
		
		ltrPresenceChanged = new CustomListener<PresenceChangedData>(){
		
					public void onAction(PresenceChangedData data)
					{
						// Update input controls state IF current TabItem is the TabITem of the user whose presence has changed
						CTabItem ti = tfFreeChats.getSelection();
						if(ti != null)
						{
							LogData ld = (LogData) ti.getData(GuiItemsDataKeys.TABITEM_LOGDATA);
							if(ld.getYouUsername().equalsIgnoreCase(data.getUser())) setInputControlsState(data.isOnline());
						}
					}};
		SimpleChatPlugin.getDefault().PresenceChanged.addListener(ltrPresenceChanged);
		
		ltrDisconnected = new CustomListener<String>(){
					public void onAction(String data)
					{
						btnSend.setEnabled(false);
						CTabItem[] tabs = tfFreeChats.getItems();
						for (int i = 0; i < tabs.length; i++)
						{
							CTabItem item = tabs[i];
							item.dispose();
						}
					}};
		SimpleChatPlugin.getDefault().Disconnected.addListener(ltrDisconnected);
		
		ltrEclLink = new CustomListener<EclipseLink>(){
					public void onAction(EclipseLink data)
					{
						txtInput.insert(data.getLinkAsString());
					}};
		SimpleChatPlugin.getDefault().RequestPasteEclipseLink.addListener(ltrEclLink);
	}

	private void unhookEvents()
	{
		if (ltrAwareness != null) this.getSite().getPage().removePartListener(ltrAwareness);
		if (ltrActivateChatView != null) SimpleChatPlugin.getDefault().ActivateChatView.removeListener(ltrActivateChatView);
		if (ltrPresenceChanged != null) SimpleChatPlugin.getDefault().PresenceChanged.removeListener(ltrPresenceChanged);
		if (ltrDisconnected != null) SimpleChatPlugin.getDefault().Disconnected.removeListener(ltrDisconnected);
		if (ltrEclLink != null) SimpleChatPlugin.getDefault().RequestPasteEclipseLink.removeListener(ltrEclLink);
	}

	public void dispose()
	{
		unhookEvents();
		toolkit.dispose();
//		for(LogData ld : SimpleChatPlugin.getDefault().htLogDataObjs.values())
//		{
//			ld.dispose(true);
//		}
//		SimpleChatPlugin plg = SimpleChatPlugin.getDefault();
//		if (plg != null) plg.closeConnection();
		super.dispose();
	}

	enum NodeClass
	{
		Conversation, Contact, Project
	};

	/*
	 * TODO: Ideer til forbedringer (kan implementeres senere)
	 * - felles "todo" liste? slik at en utvikler kan se hvilken oppgave de andre holder p med?
	 * - Kunne "dumpe" error-meldinger inn i chatten
	 */

	private void initForm(Composite parent)
	{
		parent.setLayout(new GridLayout(1, true));
		toolkit = new FormToolkit(parent.getDisplay());
		toolkit.adapt(parent);
		final Form frmEclipse = toolkit.createForm(parent);
		frmEclipse.setLayoutData(GuiUtil.createFillGridData());
		GridLayout grdClient = new GridLayout(1, true);
		frmEclipse.getBody().setLayout(grdClient);
		Composite frmClient = frmEclipse.getBody();
	
		Color clrSelected = toolkit.getColors().getColor(FormColors.SEPARATOR);
		Color clrBackground = toolkit.getColors().getBackground();

		// Right panel
		pnlRight = toolkit.createComposite(frmClient);
		pnlRight.setLayoutData(GuiUtil.createFillGridData());
		pnlRight.setLayout(new GridLayout());

//		 Inner tabfolder (chat)
		tfFreeChats = new CTabFolder(pnlRight, SWT.FLAT | SWT.TOP | SWT.BORDER);
		toolkit.adapt(tfFreeChats, true, true);
		tfFreeChats.setSelectionBackground(new Color[] { clrSelected, clrBackground }, new int[] { 33 });
		tfFreeChats.setLayoutData(GuiUtil.createFillGridData());
		tfFreeChats.addCTabFolder2Listener(new CTabFolder2Adapter()
		{
			public void close(CTabFolderEvent e)
			{
				updateInputControlsForPresence("");
			}
		});
		
		Composite compBottom = toolkit.createComposite(pnlRight, SWT.BORDER);
		GridData grdBottom = new GridData();
		grdBottom.grabExcessHorizontalSpace = true;
		grdBottom.horizontalAlignment = SWT.FILL;
		grdBottom.verticalAlignment = SWT.BOTTOM;
		compBottom.setLayoutData(grdBottom);
		compBottom.setLayout(new GridLayout(2, false));
		
		// Message input textbox
		txtInput = toolkit.createText(compBottom, "");
		txtInput.setLayoutData(GuiUtil.createFillGridData());
		txtInput.setData(INPUT_DISABLED);
		txtInput.addKeyListener(new KeyAdapter()
		{
			public void keyPressed(KeyEvent e)
			{
				if (e.keyCode == SWT.CR)
				{
					onAction_Send();
				}
			}
		});

		// Send message button
		btnSend = toolkit.createButton(compBottom, "Send", SWT.PUSH);
		btnSend.setLayoutData(GuiUtil.createFillGridData());
		GridData grdBtn = new GridData();
		grdBtn.grabExcessVerticalSpace = true;
		grdBtn.horizontalAlignment = SWT.RIGHT;
		grdBtn.verticalAlignment = SWT.FILL;
		btnSend.setLayoutData(grdBtn);
		btnSend.setEnabled(false);
		btnSend.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				onAction_Send();
			}
		});
	}

	private void createChatView(final LogData ld, TreeItem nodeContact)
	{
		ChatLogView view = (ChatLogView) nodeContact.getData(GuiItemsDataKeys.TREEITEM_CHATLOGVIEW);
		if ( (view == null) || (view.isDisposed()) )
		{
			// Create the view first
			CTabItem ti = new CTabItem(tfFreeChats, SWT.CLOSE);
			view = new ChatLogView(tfFreeChats,SWT.NONE,ld, toolkit,ti);
			
			final ChatLogView finalView = view;
			final TreeItem finalNodeContact = nodeContact;
			{
				tfFreeChats.addCTabFolder2Listener(new CTabFolder2Adapter(){
					
					public void close(CTabFolderEvent event)
					{
						CTabItem item = (CTabItem) event.item;
						if(item.getControl().equals(finalView))
						{
							if(!finalView.isDisposed())
							{
								if(!finalNodeContact.isDisposed()) finalNodeContact.setData(GuiItemsDataKeys.TREEITEM_CHATLOGVIEW, null);
								finalView.dispose();
							}
							tfFreeChats.removeCTabFolder2Listener(this); // Er dette lov da??
						}
					}
				});
				if(ld.SessionAdded == null) ld.SessionAdded = new CustomEvent<LogSession>();
				ld.SessionAdded.addListener(new CustomListener<LogSession>(){
					public void onAction(LogSession data)
					{
						finalView.appendSection(data); 
					}});
			}
			nodeContact.setData(GuiItemsDataKeys.TREEITEM_CHATLOGVIEW,view);
		}
		updateInputControlsForPresence(ld.getYouUsername());
	}

	public void setFocus()
	{
		txtInput.setFocus();
		txtInput.selectAll();
	}


	/**
	 * Called when the user sends a message (by either clicking the Send-button
	 * or by pressing Enter)
	 */
	private void onAction_Send()
	{
		// Do not allow sending if user is disconnected
		if (((String) txtInput.getData()).equalsIgnoreCase(INPUT_DISABLED)) return;
		String strText = txtInput.getText();
		if (strText.length() == 0) return;
		txtInput.setText("");
//		if (tfOutput.getSelectionIndex() == 0)
//		{
			CTabItem ti = tfFreeChats.getSelection();
			LogData ld = (LogData) ti.getData(GuiItemsDataKeys.TABITEM_LOGDATA);
			try
			{
				ld.sendMessage(strText);
			}
			catch (Exception e)
			{
				GuiUtil.handleException(e);
			}
//		}
	}


	private void updateInputControlsForPresence(String strUser)
	{
		Presence p = SimpleChatPlugin.getDefault().getConnection().getRoster().getPresence(strUser);
		setInputControlsState(!(p == null));
	}
	
	private void setInputControlsState(boolean bActivate)
	{
		btnSend.setEnabled(bActivate);
		String data = INPUT_DISABLED;
		if(bActivate) data = INPUT_ENABLED;
		txtInput.setData(data);
	}
}