package no.jabber.eclipseplugin;

import java.text.SimpleDateFormat;
import java.util.*;

import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.jivesoftware.smack.*;

/**
 * Various utility methods.
 */
public class Util
{
	// RegEx'es which are used for recognizing different patterns in a chat message
	public static String RX_HTTP_HYPERLINK = "(\\bhttp://[^ ]+\\b)"; 
	public static String RX_ECLIPSE_HYPERLINK = "(\\beclipse://[^ ]+\\b)";
	public static String[] RX_HYPERLINKS = new String[] { Util.RX_HTTP_HYPERLINK, Util.RX_ECLIPSE_HYPERLINK };
	public static String RX_HYPERLINK_REPL = "<a href='$0'>$0</a>";
	public static String RX_EMAIL = "\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*"; // Not currently in use

	// Maps between filetype and valid parameters for that filetype
	public static final Hashtable<String, String[]> FTYPE_PARAM_MAPPING = new Hashtable<String, String[]>();
	
	public static String stripUserString(String strUser)
	{
		int n = strUser.indexOf('/');
		if (n == -1) return strUser;
		else return strUser.substring(0, n);
	}
	
	
	public static String getNameFromUser(String strUser)
	{
		XMPPConnection con = SimpleChatPlugin.getDefault().getConnection();
		if(con != null)
		{
			RosterEntry re = con.getRoster().getEntry(strUser);
			if (re == null || re.getName() == null) return Util.getUsernameFromEmail(strUser);
			else return re.getName();
		}
		else return Util.getUsernameFromEmail(strUser);
	}
	
	public static String getUsernameFromEmail(String strEmail)
	{
		int n = strEmail.indexOf('@');
		if (n == -1) return strEmail;
		else return strEmail.substring(0, n);
	}
	
	public static SimpleDateFormat getDateFormatter()
	{
		return new SimpleDateFormat("dd.MMMMM yyyy HH:mm ");
	}
	
	public static boolean isStringEmpty(String strToTest, String strEmptyStringIndicator)
	{
		if(strToTest == null) return true;
		if(strToTest.equalsIgnoreCase("")) return true;
		if(strToTest.equalsIgnoreCase(strEmptyStringIndicator)) return true;
		return false;
	}
	
//	public static XMPPConnection getConnection()
//	{
//		return SimpleChatPlugin.getDefault().getConnection();
//	}
	
	public static <T> ArrayList<T> createArrayList(T s1, T s2, T s3)
	{
		ArrayList<T> al = new ArrayList<T>();
		if (s1 != null) al.add(s1);
		if (s2 != null) al.add(s2);
		if (s3 != null) al.add(s3);
		return al;
	}

	public static class CustomEvent<T>
	{
		private ArrayList<CustomListener<T>> alListeners = new ArrayList<CustomListener<T>>();


		public void addListener(CustomListener<T> listener)
		{
			alListeners.add(listener);
		}

		public void removeListener(CustomListener<T> listener)
		{
			alListeners.remove(listener);
		}
		
		public void removeAllListeners()
		{
			alListeners.clear();
		}
		
		public void fire(T data)
		{
			Iterator<CustomListener<T>> i = alListeners.iterator();
			while (i.hasNext())
			{
				i.next().onAction(data);
			}
		}
	}

	public static interface CustomListener<T>
	{
		public void onAction(T data);
	}
	
	public static class CustomEvent2<T,V>
	{
		private ArrayList<CustomListener2<T,V>> alListeners = new ArrayList<CustomListener2<T,V>>();


		public void addListener(CustomListener2<T,V> listener)
		{
			alListeners.add(listener);
		}

		public void removeListener(CustomListener2<T,V> listener)
		{
			alListeners.remove(listener);
		}
		
		public void removeAllListeners()
		{
			alListeners.clear();
		}
		
		public void fire(T data,V data2)
		{
			Iterator<CustomListener2<T,V>> i = alListeners.iterator();
			while (i.hasNext())
			{
				i.next().onAction(data,data2);
			}
		}
	}

	public static interface CustomListener2<T,V>
	{
		public void onAction(T data,V data2);
	}
}