package no.jabber.eclipseplugin.awareness;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URISyntaxException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Date;

import no.jabber.eclipseplugin.GuiUtil;
import no.jabber.eclipseplugin.Util;
import no.jabber.eclipseplugin.linking.EclipseLink;
import no.jabber.eclipseplugin.serialization.AwarenessData;
import no.jabber.eclipseplugin.serialization.AwarenessEntry;
import no.jabber.eclipseplugin.serialization.AwarenessSession;

import org.eclipse.core.resources.IWorkspace;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * The service object for using the awareness logging features.
 * 
 */
public class AwarenessService
{
	static String AWARENESS_LOG_FILENAME = "AwarenessLog";

	private AwarenessData logData;
	private String strLogFile;
	//private Browser txt;
	//private static AwarenessService instance;

	/**
	 * Constructor is private to ensure singleton behaviour. Use the static method 'create' to get an instance of this object.
	 * @param tiAwareness The CTabItem where the awareness log resides.
	 */
	public AwarenessService() throws Exception
	{
//		txt = new Browser(tiAwareness.getParent(), SWT.BORDER);
//		txt.setBackground(new Color(Display.getCurrent(), 240, 240, 240));
//		txt.addLocationListener(new BrowserLocationListener());
//		tiAwareness.setControl(txt);
//		final AwarenessService serv = this;
//		tiAwareness.addDisposeListener(new DisposeListener()
//		{
//			public void widgetDisposed(DisposeEvent e)
//			{
//				serv.saveToFile();
//			}
//		});
		
		IWorkspace ws = GuiUtil.getCurrentWorkspace();
		String strPath = ws.getRoot().getLocation().toString() + "/.logs";
		File folder = new File(strPath);
		if (!folder.exists()) folder.mkdir();

		File file = new File(strPath + "/" + AwarenessService.AWARENESS_LOG_FILENAME + ".log.xml");
		this.strLogFile = file.getAbsolutePath();
		if (file.exists()) this.loadAwarenessHistory();
		else this.logData = new AwarenessData();
//		if (this.logData != null) this.fillWithAwarenessEntries();
	}

	
	
	/**
	 * Transforms strCurrentLogAsXml to HTML and fills the Browser component. Used when loading a log file from disk.
	 * @param brw The Browser component.
	 * @param strCurrentLogAsXml The log to transform, in XML-format.
	 */
//	private void fillWithAwarenessEntries() throws Exception
//	{
//		SimpleChatPlugin plg = SimpleChatPlugin.getDefault();
//		IPath pth = new Path("/res/TransformAwareness.xslt");
//		IPath pthState = plg.getStateLocation();
//		
//		File fPin = new File(pthState.append(new Path("pin.gif")).toPortableString());
//		File fPinHover = new File(pthState.append(new Path("pin_hover.gif")).toPortableString());
//		File fUnpin = new File(pthState.append(new Path("unpin.gif")).toPortableString());
//		File fUnpinHover = new File(pthState.append(new Path("unpin_hover.gif")).toPortableString());
//		File fMinus = new File(pthState.append(new Path("-.gif")).toPortableString());
//		File fMinusHover = new File(pthState.append(new Path("-_hover.gif")).toPortableString());
//		File fPluss = new File(pthState.append(new Path("+.gif")).toPortableString());
//		File fPlussHover = new File(pthState.append(new Path("+_hover.gif")).toPortableString());
//		XmlUtil.makeSureIconsAvailable(new File[] { fPin, fPinHover, fUnpin, fUnpinHover,fMinus,fMinusHover,fPluss,fPlussHover });
//		
//
//		String strModifiedXsl = XmlUtil.changeImgSrcToLocal(pth);
//		String strHtml = XmlUtil.transformXmlToHtml(strModifiedXsl, this.getCurrentLogAsXml());
//		this.txt.setText(strHtml);
//	}

	

	/**
	 * Serializes the current log data contained in logData to valid XML.
	 */
	public String getCurrentLogAsXml()
	{
		XStream xStream = new XStream(new DomDriver());
		return xStream.toXML(logData);
	}

	/**
	 * Creates (if necessary) an instance of the awareness log. Only one instance is allowed, since there is only one awareness log.
	 * @param tiAwareness The CTabItem where the awareness log resides
	 * @return The awareness log instance
	 */
//	public static AwarenessService create(CTabItem tiAwareness) throws Exception
//	{
//		if (AwarenessService.instance == null)
//		{
//			instance = new AwarenessService(tiAwareness);

//			IWorkspace ws = GuiUtil.getCurrentWorkspace();
//			String strPath = ws.getRoot().getLocation().toString() + "/.logs";
//			File folder = new File(strPath);
//			if (!folder.exists()) folder.mkdir();
//
//			File file = new File(strPath + "/" + AwarenessService.AWARENESS_LOG_FILENAME + ".log.xml");
//			instance.strLogFile = file.getAbsolutePath();
//			if (file.exists()) instance.loadAwarenessHistory();
//			else instance.logData = new AwarenessData();
//			if (instance.logData != null) instance.fillWithAwarenessEntries();
//
//		}
//		return instance;
//	}

	/**
	 * Loads(i.e. deserializes) the awareness log from a XML file on disk.
	 * @param strLogFile The path to the log file
	 */
	private void loadAwarenessHistory() throws Exception
	{
		FileInputStream input = null;
		InputStreamReader reader = null;
		XStream xStream = new XStream(new DomDriver());
		try
		{
			input = new FileInputStream(this.strLogFile);
			reader = new InputStreamReader(input, Charset.forName("UTF-8"));
			AwarenessData logFromXml = (AwarenessData)xStream.fromXML(reader);
			logFromXml.createNewSession();
			this.logData = logFromXml;
		}
		finally
		{
				if (reader != null) reader.close();
				if (input != null) input.close();
		}
	}

	/**
	 * Returns the Javascript-string for adding an entry to the awareness log. The Javascript string must then be emitted in the browser component 
	 * @throws URISyntaxException 
	 */
//	private String getNewAwMsgJS(String strWho, String strWhere, String strWhen,String strParam1, String strParam2, String strParam3) throws URISyntaxException
//	{
//		String strWhereLoc = strWhere;
//		strWho = Util.getNameFromUser(Util.stripUserString(strWho));
//		EclipseLink link;
//		String strWhereLink = "";
//		try
//		{
//			link = new EclipseLink(strWhere,null);
//			strWhereLink = link.getLink();
//		}
//		catch (URISyntaxException ex)
//		{
//			GuiUtil.handleException(ex);
//		}
//		return "appendMessage('"+strWho+"','"+strWhereLoc+"','"+strWhereLink+"','"+strWhen+"','"+strParam1+"','"+strParam2+"','"+strParam3+"');";
//	}

	/**
	 * Adds an awareness message to the log.
	 * @param strWho Who did the message come from?
	 * @param strWhere Where did the action occur?
	 * @param strWhen When did the action occur?
	 * @param strParam1 What happened?
	 * @param strParam2 For future use
	 * @param strParam3 For future use
	 * @return The message wrapped in an AwarenessEntry object
	 */
	public AwarenessEntry addIncomingAwarenessMsg(final String strWho, final String strWhere, final String strWhen, final String strParam1, final String strParam2, final String strParam3) throws URISyntaxException
	{
//		Display.getDefault().syncExec(new Runnable()
//		{
//			public void run()
//			{
//				try
//				{
//					String strJS = getNewAwMsgJS(strWho, strWhere, strWhen,strParam1, strParam2, strParam3);
//					Thread.sleep(100);
//					txt.execute(strJS);
//				}
//				catch (Exception ex)
//				{
//					GuiUtil.handleException(ex);
//				}
//			}
//		});
		return this.logData.getLatestSession().addMessage(Util.getNameFromUser(Util.stripUserString(strWho)), new EclipseLink(strWhere,null,null), Util.getDateFormatter().format(new Date()), strParam1, strParam2, strParam3);
	}

	/**
	 * Serializes the log to XML and saves the file on disk.
	 */
	public void saveToFile()
	{
		// Remove empty sessions
		ArrayList<AwarenessSession> alSessions = this.logData.getSessions();
		ArrayList<AwarenessSession> alSessionsToRemove = new ArrayList<AwarenessSession>();
		for (int i = 0; i < alSessions.size(); i++)
		{
			AwarenessSession s = alSessions.get(i);
			if (s.getEntries().size() == 0) alSessionsToRemove.add(s);
		}

		for (int j = 0; j < alSessionsToRemove.size(); j++)
		{
			alSessions.remove(alSessionsToRemove.get(j));
		}
		FileOutputStream output = null;
		OutputStreamWriter writer = null;
		try
		{
			logData.setLastSaved(Util.getDateFormatter().format(new Date()));
			XStream xStream = new XStream(new DomDriver());
			String strXml = xStream.toXML(logData);
			output = new FileOutputStream(this.strLogFile);
			writer = new OutputStreamWriter(output, Charset.forName("UTF-8"));
			writer.write(strXml);
			writer.close();
		}
		catch(Exception e)
		{
			GuiUtil.handleException(e); // Handle the exception here since we are not allowed to throw it
		}
		finally
		{
			try
			{
				if (writer != null) writer.close();
				if (output != null) output.close();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}



	public void dispose()
	{
		this.saveToFile();
	}
}
