package no.jabber.eclipseplugin.dialogs;


import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;

import no.jabber.eclipseplugin.GuiUtil;
import no.jabber.eclipseplugin.Util;
import no.jabber.eclipseplugin.XmlUtil;
import no.jabber.eclipseplugin.Util.CustomEvent;
import no.jabber.eclipseplugin.Util.CustomListener;
import no.jabber.eclipseplugin.dialogs.PopupHyperlink.HyperlinkType;
import no.jabber.eclipseplugin.linking.EclipseLink;
import no.jabber.eclipseplugin.serialization.LogEntry;
import no.jabber.eclipseplugin.serialization.LogSession;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.FormText;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.Section;

public class ChatSection
{

	private Section SectionControl;
	private Label lblHeader;
	
	private Color clrInactive; 
	private Color clrActive;
	private FormToolkit toolkit;
	private final LogSession session;
	private ArrayList<IMarker> alMarkers = new ArrayList<IMarker>();
	
	public CustomEvent<String> ReflowParent = new CustomEvent<String>();
	public CustomEvent<ChatSection> ScrollToSection = new CustomEvent<ChatSection>();
	public CustomEvent<ChatSection> ScrollSectionToBottom = new CustomEvent<ChatSection>();
	private Hyperlink hlGotoLocation;
	private Composite compDesc;
	
	public ChatSection(Composite parent, int style, Color clrActive, Color clrInactive, FormToolkit toolkit,final LogSession session)
	{
		SectionControl = new Section(parent,style);
		this.session = session;
		this.toolkit = toolkit;
		this.clrInactive = clrInactive;
		this.clrActive = clrActive;
		lblHeader = new Label(SectionControl,SWT.NONE);
		SectionControl.setTextClient(lblHeader);
		
		final Composite sectionClient = toolkit.createComposite(SectionControl);
		GridLayout grid = new GridLayout();
		grid.numColumns = 2;
		sectionClient.setLayout(grid);
		for (Iterator iterator = session.getMessages().iterator(); iterator.hasNext();)
		{
			LogEntry entry = (LogEntry) iterator.next();
			appendEntryToSection(sectionClient,entry);
		}
		SectionControl.setClient(sectionClient);
		this.setMarker();
		SectionControl.getTextClient().addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e)
			{
				// Update the data model...NOT the GUI
				// Because we have event handlers that modifies the tree on active changed as well as this GUI component
				session.setActive(!session.isActive());
			}
		});
		SectionControl.addMouseListener(new MouseAdapter(){
			public void mouseDoubleClick(MouseEvent e)
			{
				session.setActive(!session.isActive());
			}
		});
		if(session.ContextChanged == null) session.ContextChanged = new CustomEvent<String>();
		if(session.ProjectChanged == null) session.ProjectChanged = new CustomEvent<String>();
		if(session.MessageAdded == null) session.MessageAdded = new CustomEvent<LogEntry>();
		if(session.ActiveStateChanged == null) session.ActiveStateChanged = new CustomEvent<Boolean>();
		if(session.ContextLocationChanged == null) session.ContextLocationChanged = new CustomEvent<String>();
		
		final CustomListener<String> ltrContext = new CustomListener<String>(){
			public void onAction(String strNewContext)
			{
				setHeader();
//				txtContext.setText(strNewContext);
			}};
		final CustomListener<String> ltrProject = new CustomListener<String>(){
				public void onAction(String data)
				{
					setHeader();
				}};
		final CustomListener<LogEntry> ltrMessage = new CustomListener<LogEntry>(){
			public void onAction(LogEntry data)
			{
				if(!session.isActive()) boldifyHeader(true);
				appendEntryToSection(sectionClient, data);
				if(session.isActive()) fireScrollToBottomEvent();
				setMarker();
			}};
		final CustomListener<Boolean> ltrActivity = new CustomListener<Boolean>(){
			public void onAction(Boolean data)
			{
				// UPDATE the GUI!!
				setActive(data.booleanValue());
			}};
		final CustomListener<String> ltrContextLoc = new CustomListener<String>(){
			public void onAction(String strNewLoc)
			{
				hlGotoLocation.setHref(strNewLoc);
				//hlGotoLocation.setText(getGotoLocationString(strNewLoc));
				setGotoLocationString(hlGotoLocation, strNewLoc);
			}
		};
		// Add the listeners
		session.ContextChanged.addListener(ltrContext);
		session.ProjectChanged.addListener(ltrProject);
		session.MessageAdded.addListener(ltrMessage);
		session.ActiveStateChanged.addListener(ltrActivity);
		session.ContextLocationChanged.addListener(ltrContextLoc);
		
		SectionControl.addDisposeListener(new DisposeListener(){
			public void widgetDisposed(DisposeEvent e)
			{
				// Only remove listeners in this class
				session.ActiveStateChanged.removeListener(ltrActivity);
				session.ProjectChanged.removeListener(ltrProject);
				session.ContextChanged.removeListener(ltrContext);
				session.MessageAdded.removeListener(ltrMessage);
				session.ContextLocationChanged.removeListener(ltrContextLoc);
				
				// Remove all markers!
				for(IMarker m : alMarkers)
				{
					
					try
					{
						m.delete();
					}
					catch (CoreException ex)
					{
						// TODO Auto-generated catch block
						ex.printStackTrace();
					}
				}
			}});
		
		SectionControl.addExpansionListener(new ExpansionAdapter() {
			public void expansionStateChanged(ExpansionEvent e) 
			{
				ReflowParent.fire("reflow");
			}
		});
		this.setHeader();
		
		
	}
	
	private void setMarker()
	{
		String strLoc = session.getContextLocation();
		// Make code for implmenting a bookmark at the correct location in for the file containing chat!
		if(strLoc != null)
		{
			try
			{
				EclipseLink el = new EclipseLink(strLoc);
				IResource res = el.getResource();
				if(res != null)
				{
//					IMarker mrk = res.createMarker("chatmarker");
					IMarker mrk = res.createMarker(IMarker.BOOKMARK);
					mrk.setAttribute(IMarker.MESSAGE,session.getMessagesAsText());
					if(res instanceof IFile)
					{
						mrk.setAttribute(IMarker.LINE_NUMBER, 1);
					}
					
//					SimpleMarkerAnnotation annot = new SimpleMarkerAnnotation(mrk);
					
//					((ITextEditor) GuiUtil.getCurrentEditor()).getDocumentProvider().getAnnotationModel(element)
//					annot.
					
					alMarkers.add(mrk);
				}
			}
			catch (URISyntaxException ex)
			{
				// TODO Auto-generated catch block
				ex.printStackTrace();
			}
			catch (CoreException ex)
			{
				// TODO Auto-generated catch block
				ex.printStackTrace();
			}
		}
	}
	
		
	private void setGotoLocationString(Hyperlink hl, String strNewLoc)
	{
		if(Util.isStringEmpty(strNewLoc, LogSession.DEFAULT_CONTEXT_LOCATION))
		{
			hl.setHref(null);
			hl.setText("Goto location "+ LogSession.DEFAULT_CONTEXT_LOCATION);
			hl.setToolTipText(LogSession.DEFAULT_CONTEXT_LOCATION + " (CTRL + Click to edit)");
		}
		else
		{
			hl.setHref(strNewLoc);
			hl.setText("Goto location(...)");
			hl.setToolTipText(strNewLoc +  " (CTRL + Click to edit)");
		}
	}
	
	private void appendEntryToSection(Composite sectionClient, LogEntry entry)
	{
		Control[] aSectionControls = sectionClient.getChildren();
		final FormText txtSection;
		if((aSectionControls == null ) || (aSectionControls.length == 0))
		{
			// create FormText control
			txtSection = toolkit.createFormText(sectionClient,true);
			txtSection.setColor("clrMe", new Color(txtSection.getDisplay(),toolkit.getColors().getSystemColor(SWT.COLOR_BLUE)));
			txtSection.addHyperlinkListener(new HyperlinkAdapter(){
				public void linkActivated(HyperlinkEvent e)
				{
					String strHref = e.getHref().toString();
					if(EclipseLink.isEclipseLink(strHref))
					{
						EclipseLink linkEclipse = null; 
						try
						{
							linkEclipse = new EclipseLink(strHref);
						}
						catch (URISyntaxException ex)
						{
							GuiUtil.handleException(ex);
						}
						if(linkEclipse.linksToWorkspaceResource())
						{
							try
							{
								GuiUtil.openEditorForEclipseLink(linkEclipse);
							}
							catch (Exception ex)
							{
								GuiUtil.handleException(ex);
							}
						}
					}
				}});
		}
		else
		{
			txtSection = (FormText) aSectionControls[0];
		}
		String strOldText = null;
		if(txtSection.getData() == null) strOldText = "<form>";
		else 
		{
			strOldText = (String) txtSection.getData();
			// remove last </form> tag
			strOldText = strOldText.substring(0, strOldText.length()-7);
		}
		String strMsg = entry.getMsg();
		strMsg = XmlUtil.stringToHTMLString(strMsg);
		// Scan message for eclipse-links and replace them with <a> tags
		String strMsg2 = strMsg.replaceAll(Util.RX_ECLIPSE_HYPERLINK, Util.RX_HYPERLINK_REPL);
		if(!strMsg2.equalsIgnoreCase(strMsg)) 
		{
			// TODO: Remove "eclipse://workspace" from inside <a> tags
			strMsg = strMsg2;
		}
		String strFrom = entry.getFrom();
		String strSpanStart = "";
		String strSpanEnd = "";
		if(strFrom.equalsIgnoreCase("$me"))
		{
			// Make text blue
			strSpanStart = "<span color='clrMe'>";
			strSpanEnd = "</span>";
		}
		strOldText = strOldText + "<p vspace='true'>" + strSpanStart + strFrom +  ": "+ strSpanEnd + strMsg +"</p>";
		strOldText = strOldText + "</form>";
		txtSection.setText(strOldText, true, true);
		txtSection.setData(strOldText);
		this.ReflowParent.fire("reflow");
		
		
		
	}
	
	private void setHeader()
	{
		SectionControl.setText("Project: "+session.getProject()+", Context: "+session.getContext()+" from "+session.getLatestTimestamp());
		SectionControl.redraw();
	}
	private void setActive(boolean bActivate)
	{
		Color clrTheColor = this.clrInactive;
		if(bActivate) 
		{
			this.boldifyHeader(false);
			clrTheColor = this.clrActive;
			SectionControl.setExpanded(true);
			this.ScrollToSection.fire(this);
		}
		SectionControl.setBackground(clrTheColor);
		SectionControl.getTextClient().setBackground(clrTheColor);
	}
	


	private void boldifyHeader(boolean bMakeBold)
	{
		this.lblHeader.setFont(GuiUtil.boldifyFont(bMakeBold, this.lblHeader.getFont()));
	}

	private GridData createFillVerticalLeftHor()
	{
		GridData grd = new GridData();
		grd.verticalAlignment = SWT.FILL;
		grd.horizontalAlignment = SWT.LEFT;
		return grd;
	}
	
	public void createInputPanel()
	{
		compDesc = toolkit.createComposite(SectionControl);
		GridLayout grdDesc = new GridLayout(3,false);
		grdDesc.horizontalSpacing = 5;
		grdDesc.verticalSpacing = 0;
		compDesc.setLayout(grdDesc);
		hlGotoLocation = toolkit.createHyperlink(compDesc, "", SWT.NONE);
		hlGotoLocation.setLayoutData(createFillVerticalLeftHor());
		hlGotoLocation.addHyperlinkListener(new PopupHyperlink(session,hlGotoLocation,HyperlinkType.SetLocation));
		hlGotoLocation.setHref(session.getContextLocation());
		setGotoLocationString(hlGotoLocation,session.getContextLocation());
		
		final Hyperlink hlAdditional = toolkit.createHyperlink(compDesc, "More info..", SWT.NONE);
		hlAdditional.setToolTipText("Here you can see additional information related to the context (CTRL + Click to edit)");
		hlAdditional.setLayoutData(createFillVerticalLeftHor());
		hlAdditional.addHyperlinkListener(new PopupHyperlink(session,hlAdditional,HyperlinkType.SetAdditionalInfo));
		
//		new HyperlinkAdapter(){
//
//			AdditionalInfoDialog dlg;
//			
//			private void openDlg()
//			{
//				Shell sh = new Shell( SWT.ON_TOP | SWT.TOOL | SWT.RESIZE);
//				dlg = new AdditionalInfoDialog(sh,SWT.NONE);
//				dlg.InfoUpdated.addListener(new CustomListener<String>(){
//					public void onAction(String strNewInfo)
//					{
//						session.setMoreInfo(strNewInfo);
//					}});
//				Point ptHlLoc = Display.getCurrent().map(hlAdditional.getParent(), null, hlAdditional.getLocation()); 
//				dlg.open(ptHlLoc.x,ptHlLoc.y+hlAdditional.getBounds().height,session.getMoreInfo(),false,200,200);
//			}
//			
//			public void linkEntered(HyperlinkEvent e)
//			{
//				if( (dlg == null) || (dlg.isClosed())) openDlg();
//			}
//
//			public void linkExited(HyperlinkEvent e)
//			{
//				if((dlg != null) && (!dlg.bDontCloseOnExit) )
//				{
//					dlg.close();
//					dlg = null;
//				}
//			}
//
//			public void linkActivated(HyperlinkEvent e)
//			{
//				if( (dlg == null) || (dlg.isClosed()) ) 
//				{
//					openDlg();
//					dlg.setFocus();
//				}
//				dlg.bDontCloseOnExit = true;
//			}
//			}
		
		Hyperlink hlToBottom = toolkit.createHyperlink(compDesc, "Scroll to bottom", SWT.NONE);
		hlToBottom.setLayoutData(createFillVerticalLeftHor());
		hlToBottom.addHyperlinkListener(new HyperlinkAdapter(){
			public void linkActivated(HyperlinkEvent e)
			{
				fireScrollToBottomEvent();
			}
		});
		
		toolkit.paintBordersFor(compDesc);
		SectionControl.setDescriptionControl(compDesc);
	}

	private void fireScrollToBottomEvent()
	{
		ScrollSectionToBottom.fire(this);
	}
	
	public LogSession getSession()
	{
		return this.session;
	}

	public Section getSectionControl()
	{
		return this.SectionControl;
	}
}
