package no.jabber.eclipseplugin.dialogs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import no.jabber.eclipseplugin.projectmanagement.Contact;
import no.jabber.eclipseplugin.projectmanagement.Project;
import no.jabber.eclipseplugin.projectmanagement.ProjectConfig;

import org.eclipse.core.resources.IProject;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Table;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableLayout;


public class ConfigProjectDialog extends Dialog
{
	private Hashtable<String,Project> htConfiguredProjs = new Hashtable<String, Project>();  //  @jve:decl-index=0:
//	private ProjectConfig cfgData;  //  @jve:decl-index=0:
	private Collection<Contact> collAllContacts;
	private IProject[] aWorkspaceProjects;
	private boolean bResult = false;
	private boolean bDone = false;
	
	private Shell sShell = null;  //  @jve:decl-index=0:visual-constraint="16,8"
	private Composite compLeft = null;
	private Composite compRight = null;
	private Label lblProjsHeader = null;
	private List lstProjects = null;
	private Text txtCommonName = null;
	private Text txtLocalMapping = null;
	private Composite compPropGrid = null;
	private Label lblCommon = null;
	private Label lblLocal = null;
	private Composite compBottom = null;
	private Composite compClient = null;
	private Button btnCancel = null;
	private Button btnOK = null;
	private Table tblProjectMembers = null;
	private TableColumn tbCol;
	private TableLayout layTbl;
	
	public ConfigProjectDialog(Shell parent, Collection<Contact> collAllContacts, IProject[] aAvailableProjects)
	{
		this(parent,SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
//		this.cfgData = cfg;
		this.aWorkspaceProjects = aAvailableProjects;
		this.collAllContacts = collAllContacts;
		htConfiguredProjs = ProjectConfig.getConfig().getAllProjects();
		this.createSShell();
		this.fillData();
		// Select first project in list
		if(lstProjects.getItems().length > 0)
		{
			lstProjects.select(0);
			this.onLstProjects_Select();
		}
	}

	private ConfigProjectDialog(Shell parent, int style)
	{
		super(parent, style);
	}

	public boolean open()
	{
		this.sShell.open();
		Display display = this.getParent().getDisplay();
		while (!this.bDone && !sShell.isDisposed())
		{
			if (!display.readAndDispatch()) display.sleep();
		}
		return this.bResult;
	}

	private Collection<Contact> getCheckedMembers()
	{
		ArrayList<Contact> alChecked = new ArrayList<Contact>();
		TableItem[] aAllContacts = tblProjectMembers.getItems();
		for (int i = 0; i < aAllContacts.length; i++)
		{
			TableItem item = aAllContacts[i];
			if(item.getChecked()) alChecked.add((Contact)item.getData());
		}
		return alChecked;
	}
	
	private void addChange()
	{
//		 Put changed data inside changed project hashtable
		if(strLastProject != null)
		{
			Project projNewCfg = new Project();
			projNewCfg.setCommonName(txtCommonName.getText());
			projNewCfg.setLocalMapping(txtLocalMapping.getText()); // Not necessary
			projNewCfg.setMembers(getCheckedMembers());
			htProjectsChanged.put(projNewCfg.getLocalMapping(),projNewCfg);
		}
	}
	
	private Hashtable<String, Project> htProjectsChanged = new Hashtable<String,Project>();  //  @jve:decl-index=0:
	private String strLastProject = null; // The local mapping for last selected project  //  @jve:decl-index=0:
	
	private void onLstProjects_Select()
	{
		this.addChange();
		
		// Clear data before loading new project
		txtLocalMapping.setText("");
		txtCommonName.setText("");
		// Uncheck all contacts
		TableItem[] aAllContacts = tblProjectMembers.getItems();
		for (int i = 0; i < aAllContacts.length; i++)
		{
			TableItem item = aAllContacts[i];
			item.setChecked(false);
		}
		
		
		int n = lstProjects.getSelectionIndex();
		IProject prjLocal = aWorkspaceProjects[n];
		String strLocalMapping = prjLocal.getLocation().toOSString();
		strLastProject = strLocalMapping;
		Project prjConfigured = htProjectsChanged.get(strLocalMapping);
		if(prjConfigured == null)  prjConfigured = htConfiguredProjs.get(strLocalMapping);
		if(prjConfigured == null) txtLocalMapping.setText(strLocalMapping);
		else 
		{
			txtLocalMapping.setText(prjConfigured.getLocalMapping());
			txtCommonName.setText(prjConfigured.getCommonName());
			Collection<Contact> collPrjMembers = prjConfigured.getMembers();
			for (Iterator iter = collPrjMembers.iterator(); iter.hasNext();)
			{
				Contact c = (Contact) iter.next();
				this.findAndCheckMember(c);
			}
		}
	}
	
	private void findAndCheckMember(Contact contact)
	{
		TableItem[] aAllMembers = tblProjectMembers.getItems();
		for (int i = 0; i < aAllMembers.length; i++)
		{
			TableItem item = aAllMembers[i];
			Contact contInItem = (Contact) item.getData();
			if(contInItem.getUserName().equalsIgnoreCase(contact.getUserName()))
			{
				item.setChecked(true);
				break;
			}
		}
	}

	private void fillData()
	{
		// Fill project list
		if( (this.aWorkspaceProjects != null) && (this.aWorkspaceProjects.length > 0) )
		{
			for (int i = 0; i < this.aWorkspaceProjects.length; i++)
			{
				IProject proj = this.aWorkspaceProjects[i];
				lstProjects.add(proj.getName());				
			}
		}
		// Fill contact list
		for (Iterator iter = this.collAllContacts.iterator(); iter.hasNext();)
		{
			Contact c = (Contact) iter.next();
			TableItem ti = new TableItem(tblProjectMembers,SWT.NONE);
			ti.setText(c.getFriendlyName());
			ti.setData(c);
		}
	}

	/**
	 * This method initializes sShell	
	 *
	 */
	private void createSShell()
	{
		GridLayout gridLayout = new GridLayout(2, true);
		gridLayout.makeColumnsEqualWidth = false;
		gridLayout.numColumns = 1;
		sShell = new Shell(this.getParent(),this.getStyle());
		//sShell = new Shell(); 
		sShell.setText("Configure projects");
		createCompClient();
		sShell.setLayout(gridLayout);
		createCompBottom();
		sShell.setSize(new Point(421, 317));
	}

	/**
	 * This method initializes compLeft	
	 *
	 */
	private void createCompLeft()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		GridData gridData8 = new GridData();
		gridData8.horizontalAlignment = GridData.FILL;
		gridData8.grabExcessVerticalSpace = true;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.verticalAlignment = GridData.FILL;
		compLeft = new Composite(compClient, SWT.NONE);
		compLeft.setLayout(new GridLayout());
		compLeft.setLayoutData(gridData1);
		
	
		lblProjsHeader = new Label(compLeft, SWT.NONE);
		lblProjsHeader.setText("Available projects");
		lstProjects = new List(compLeft, SWT.NONE);
		lstProjects.setLayoutData(gridData8);
		lstProjects.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter()
		{
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
			{
				onLstProjects_Select();
			}
		});
		
	}

	/**
	 * This method initializes compRight	
	 *
	 */
	private void createCompRight()
	{
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		GridLayout gridLayout6 = new GridLayout();
		gridLayout6.marginHeight = 0;
		gridLayout6.numColumns = 1;
		GridData gridData11 = new GridData();
		gridData11.horizontalAlignment = GridData.FILL;
		gridData11.grabExcessHorizontalSpace = true;
		gridData11.grabExcessVerticalSpace = true;
		gridData11.verticalAlignment = GridData.FILL;
		compRight = new Composite(compClient, SWT.NONE);
		createCompPropGrid();
		compRight.setLayoutData(gridData11);
		compRight.setLayout(gridLayout6);
		tblProjectMembers = new Table(compRight, SWT.MULTI | SWT.CHECK);
		tblProjectMembers.setHeaderVisible(true);
		tblProjectMembers.setLayoutData(gridData4);
		tblProjectMembers.setLinesVisible(true);
		layTbl = new TableLayout();
		layTbl.addColumnData(new ColumnWeightData(100,false));
		tblProjectMembers.setLayout(layTbl);
		tbCol = new TableColumn(tblProjectMembers,SWT.CENTER);
		tbCol.setText("Check for project membership");
	}

	/**
	 * This method initializes compPropGrid	
	 *
	 */
	private void createCompPropGrid()
	{
		GridData gridData10 = new GridData();
		gridData10.horizontalAlignment = GridData.FILL;
		gridData10.verticalAlignment = GridData.CENTER;
		GridData gridData3 = new GridData();
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.verticalAlignment = GridData.CENTER;
		gridData3.horizontalAlignment = GridData.FILL;
		GridData gridData2 = new GridData();
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		gridData2.horizontalAlignment = GridData.FILL;
		GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 2;
		gridLayout1.makeColumnsEqualWidth = false;
		compPropGrid = new Composite(compRight, SWT.NONE);
		compPropGrid.setLayout(gridLayout1);
		compPropGrid.setLayoutData(gridData10);
		lblLocal = new Label(compPropGrid, SWT.NONE);
		lblLocal.setText("Local mapping");
		txtLocalMapping = new Text(compPropGrid, SWT.BORDER);
		txtLocalMapping.setEditable(false);
		txtLocalMapping.setLayoutData(gridData2);
		lblCommon = new Label(compPropGrid, SWT.NONE);
		lblCommon.setText("Common name");
		txtCommonName = new Text(compPropGrid, SWT.BORDER);
		txtCommonName.setLayoutData(gridData3);
	}

//	public ProjectConfig getCfgData()
//	{
//		return this.cfgData;
//	}
	
	private void onBtnOk_Click()
	{
		// Add possible changes
		this.addChange(); 
		
		// Add all projects to the hashtable in case some projects have not been configured in this session
		int nChangedProjs = htProjectsChanged.size();
		if((htProjectsChanged != null) && (nChangedProjs > 0) )
		{
			Enumeration<String> enKeys = htConfiguredProjs.keys();
			while(enKeys.hasMoreElements())
			{
				String strKey = enKeys.nextElement();
				if(htProjectsChanged.get(strKey) == null)
					htProjectsChanged.put(strKey, htConfiguredProjs.get(strKey));
			}
			// Set the configured projects
//			if(this.cfgData == null) this.cfgData = new ProjectConfig(); // TODO: Dette er feil. M i alle fall koble opp project events etter dette!
			ProjectConfig.getConfig().setAllProjects(htProjectsChanged.values());
			this.bResult = true;
		} else this.bResult = false; // No changes == Cancel
		bDone = true;
		sShell.dispose();
	}
	
	private void onBtnCancel_Click()
	{
		// Do nothing, close form
		bResult = false;
		bDone = true;
		sShell.dispose();
	}

	/**
	 * This method initializes compBottom	
	 *
	 */
	private void createCompBottom()
	{
		GridData gridData14 = new GridData();
		gridData14.widthHint = 65;
		gridData14.heightHint = 23;
		GridData gridData13 = new GridData();
		gridData13.heightHint = 23;
		gridData13.widthHint = 65;
		GridData gridData12 = new GridData();
		gridData12.horizontalAlignment = GridData.END;
		gridData12.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout5 = new GridLayout();
		gridLayout5.numColumns = 2;
		compBottom = new Composite(sShell, SWT.NONE);
		compBottom.setLayout(gridLayout5);
		compBottom.setLayoutData(gridData12);
		btnOK = new Button(compBottom, SWT.NONE);
		btnOK.setText("OK");
		btnOK.setLayoutData(gridData13);
		btnOK.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter()
		{
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
			{
				onBtnOk_Click();
			}
		});
		btnCancel = new Button(compBottom, SWT.NONE);
		btnCancel.setText("Cancel");
		btnCancel.setLayoutData(gridData14);
		btnCancel.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter()
		{
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
			{
				onBtnCancel_Click();
			}
		});
	}
	/**
	 * This method initializes compClient	
	 *
	 */
	private void createCompClient()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		GridLayout gridLayout4 = new GridLayout();
		gridLayout4.numColumns = 2;
		compClient = new Composite(sShell, SWT.NONE);
		compClient.setLayout(gridLayout4);
		createCompLeft();
		compClient.setLayoutData(gridData);
		createCompRight();
	}
}  //  @jve:decl-index=0:
