package no.jabber.eclipseplugin.popup.actions;

import java.net.URISyntaxException;

import no.jabber.eclipseplugin.GuiUtil;
import no.jabber.eclipseplugin.linking.UnsupportedJavaElement;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;

/**
 * This action is executed when a user wants to create a link from a selection in an editor using a right-click menu.
 * For now, the selection must be a valid java-element. 
 */
public class MakeLinkFromTextAction implements IEditorActionDelegate 
{
	String strLastSelection;
	
	public void setActiveEditor(IAction action, IEditorPart targetEditor){}

	public void run(IAction action)
	{
		if( (strLastSelection != null) && (strLastSelection.length() > 0) )
		{
			int nFirstWhiteSpace = strLastSelection.indexOf(" ");
			if(nFirstWhiteSpace == -1) 
			{
				IJavaElement elem = GuiUtil.findJavaElementMatch(strLastSelection);
				if(elem != null)
				{
					try
					{
						GuiUtil.createLinkAndInsert(elem);
					}
					catch (URISyntaxException ex)
					{
						GuiUtil.handleException(ex, "There was an error in the link.");
					}
					catch (UnsupportedJavaElement ex)
					{
						GuiUtil.handleUnsupportedJavaElementException(ex);
					}
				}
				else
				{
					GuiUtil.handleException(new Exception(), "The selection was not a Java element.");
				}
			}
			else
			{
				GuiUtil.handleException(new Exception(), "The selection can't have any whitespaces.");
			}
		}
		else
		{
			GuiUtil.handleException(new Exception(), "Not a valid selection.");
		}
	}

	public void selectionChanged(IAction action, ISelection selection)
	{
		if( (selection != null) && (selection instanceof ITextSelection) )
		{
			ITextSelection sel = (ITextSelection) selection;
			strLastSelection = sel.getText();
		}
	}
}
