package no.jabber.eclipseplugin.projectmanagement;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

import no.jabber.eclipseplugin.SimpleChatPlugin;
import no.jabber.eclipseplugin.Util.CustomEvent;

public class ProjectConfig
{
	private transient static String SIMPLE_CHAT_CFG_FILE = "./.simpleChatProjects.xml";
	private ArrayList<Project> Projects = new ArrayList<Project>();
	private transient static ProjectConfig instance;
	
	private ProjectConfig(){}

	public static ProjectConfig getConfig()
	{
		if (ProjectConfig.instance == null) 
		{
			ProjectConfig.instance = ProjectConfig.readSimpleChatConfigFile();
			instance.ConfigurationUpdated = new CustomEvent<Collection<Project>>();
		}
		return ProjectConfig.instance;
	}

	private static IPath getConfigPath()
	{
		SimpleChatPlugin plg = SimpleChatPlugin.getDefault();
		IPath pthState = plg.getStateLocation();
		return pthState.append(SIMPLE_CHAT_CFG_FILE);
	}
	
	public void saveConfig()
	{
		ProjectConfig.saveSimpleChatConfigFile(this);
	}
	
	private static ProjectConfig readSimpleChatConfigFile()
	{
		// Put config file in meta-folder for plug-in
		IPath pthCfg = getConfigPath();
		if(pthCfg.toFile().exists())
		{
			FileInputStream input = null;
			try
			{
				input = new FileInputStream(pthCfg.toOSString());
			}
			catch (FileNotFoundException ex){}
			InputStreamReader reader = new InputStreamReader(input, Charset.forName("UTF-8"));
			XStream xs = new XStream(new DomDriver());
			ProjectConfig pc = (ProjectConfig) xs.fromXML(reader);
			return pc;
		}
		else return new ProjectConfig(); // TODO: Return empty ProjectConfig class?
	}
	
	private static void saveSimpleChatConfigFile(ProjectConfig cfg)
	{
		XStream xs = new XStream(new DomDriver());
		String strXml = xs.toXML(cfg);
		FileOutputStream output = null;
		OutputStreamWriter writer = null;
		IPath pthCfg = getConfigPath();
		try
		{
			output = new FileOutputStream(pthCfg.toOSString());
			writer = new OutputStreamWriter(output, Charset.forName("UTF-8"));
			writer.write(strXml);
			writer.close();
			if(cfg.ConfigurationUpdated == null) cfg.ConfigurationUpdated = new CustomEvent<Collection<Project>>();
			// TODO: Implement method that only returns changed projects
			cfg.ConfigurationUpdated.fire(cfg.getProjects());
		}
		catch(Exception ex)
		{
			// TODO: Handle exception
			ex.printStackTrace();
		}
	}
	
	public transient CustomEvent<Collection<Project>> ConfigurationUpdated = new CustomEvent<Collection<Project>>();
	//private transient Hashtable<String,Project> htProjects = new Hashtable<String, Project>();
	
	public Hashtable<String,Project> getAllProjects()
	{
		Hashtable<String,Project> ht = new Hashtable<String, Project>();
		for (Iterator iter = Projects.iterator(); iter.hasNext();)
		{
			Project prj = (Project) iter.next();
			ht.put(prj.getLocalMapping(),prj);
		}
		return ht;
	}

	public void setAllProjects(Collection<Project> collProjs)
	{
		this.Projects.clear();
		this.Projects.addAll(collProjs);
//		if(this.ConfigurationUpdated == null) this.ConfigurationUpdated = new CustomEvent<Collection<Project>>();
//		this.ConfigurationUpdated.fire(collProjs);
	}

	public Project getProjectByName(String strPrjName)
	{
		for (Iterator iter = Projects.iterator(); iter.hasNext();)
		{
			Project prj = (Project) iter.next();
			if(prj.getCommonName().equalsIgnoreCase(strPrjName)) return prj;
		}
		return null;
	}

	public Collection<Project> getProjects()
	{
		return this.Projects;
	}

	public Project resolveProjectFromIProject(IProject prj)
	{
		for (Iterator iter = Projects.iterator(); iter.hasNext();)
		{
			Project p1 = (Project) iter.next();
			if(p1.getLocalMapping().equalsIgnoreCase(prj.getLocation().toOSString())) return p1;
		}
		return null;
	}
}
