package no.jabber.eclipseplugin.serialization;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TreeItem;
import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.packet.Message;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

import no.jabber.eclipseplugin.GuiUtil;
import no.jabber.eclipseplugin.SimpleChatPlugin;
import no.jabber.eclipseplugin.Util;
import no.jabber.eclipseplugin.Util.CustomEvent;
import no.jabber.eclipseplugin.Util.CustomListener;
import no.jabber.eclipseplugin.projectmanagement.Project;


/**
 * The root class for chatlog data. Used for serializing and deserializing chat log.
 */
@SuppressWarnings("unused")
public class LogData
{
	// TODO: Rydd!
	// Fields and methods related to the data model
	//*********************************************
	public transient static String DEFAULT_ME_ID = "$me";
	private String Me;
	private String You;
	private String YouUsername;
	private ArrayList<LogSession> Sessions = new ArrayList<LogSession>();
	private String Created;
	private String LastSaved;
	
	private LogData(String strMe, String strYou, String strContext, String strContextLocation, String strProject, String strYouUsername)
	{
		this.Me = strMe;
		this.You = strYou;
		this.YouUsername = strYouUsername;
		this.Created = Util.getDateFormatter().format(new Date());
		this.createNewSession(strContext,strContextLocation,strProject);
	}
	
	public LogSession createNewSession(String strContext, String strContextLocation, String strProject)
	{
		final LogSession s = new LogSession(this,strContext,strContextLocation,strProject);
		Sessions.add(s);
		if(this.SessionAdded == null) this.SessionAdded = new CustomEvent<LogSession>();
		Display.getDefault().syncExec(new Runnable(){

			public void run()
			{
				SessionAdded.fire(s);
			}});
		initSessionEvents(s);
		return s;
	}
	
	public String getMe()
	{
		return this.Me;
	}

	public void setMe(String strMe)
	{
		this.Me = strMe;
	}

	public String getYou()
	{
		return this.You;
	}

	public void setYou(String strYou)
	{
		this.You = strYou;
	}

	public String getLastSaved()
	{
		return this.LastSaved;
	}

	public void setLastSaved(String lastSaved)
	{
		this.LastSaved = lastSaved;
	}

	public ArrayList<LogSession> getSessions()
	{
		return Sessions;
	}

	public void setSessions(ArrayList<LogSession> sessions)
	{
		Sessions = sessions;
	}

	public LogSession getLatestSession()
	{
		return this.Sessions.get(Sessions.size() - 1);
	}
	
	public LogSession getSessionByContext(String strContext,String strProject)
	{
		LogSession theSession = null;
//		Iterator<LogSession> iter = Sessions.iterator(); iter.hasNext();
		for (LogSession sess : Sessions)
		{
			if((sess.getContext().equalsIgnoreCase(strContext)) && (sess.getProject().equalsIgnoreCase(strProject)))
			{
				theSession = sess;
				break;
			}
		}
		if(theSession == null)
		{
			theSession = this.createNewSession(strContext,LogSession.DEFAULT_CONTEXT_LOCATION,strProject);
		}
		theSession.setParent(this);
		return theSession;
	}
	
	public String getYouUsername()
	{
		return this.YouUsername;
	}

	public void setYouUsername(String youUsername)
	{
		this.YouUsername = youUsername;
	}

	
	// Fields and methods related to behaviour
	//****************************************
	private transient LogSession sessionActive;
	private transient Chat chat;
	private transient String strLogFile;
	public transient Util.CustomEvent<LogSession> SessionAdded = new CustomEvent<LogSession>();
	public transient Util.CustomEvent<LogSession> SessionDataChanged = new CustomEvent<LogSession>();
	public transient Util.CustomEvent<Boolean> Disposed = new CustomEvent<Boolean>();
	private transient TreeItem nodeContact;
	
	private static File getLogFilePath(String strUser)
	{
		IWorkspace ws = GuiUtil.getCurrentWorkspace();
		String strPath = ws.getRoot().getLocation().toString() + "/.logs";
		File folder = new File(strPath);
		if (!folder.exists()) folder.mkdir();
		File file = new File(strPath + "/" + strUser + ".log.xml");
		return file;
	}
	
	public static LogData create(String strUser) throws IOException
	{
		File file = getLogFilePath(strUser);
		String strLogFile = file.getAbsolutePath();
		String strName = Util.getNameFromUser(strUser);
		LogData ld = null;
		if (file.exists()) 
		{
			ld = loadFromFile(strLogFile);
			// Assert that there is a default context
			if (ld.getSessionByContext(LogSession.DEFAULT_CONTEXT_NAME, LogSession.DEFAULT_PROJECT_NAME) == null) ld.createNewSession(LogSession.DEFAULT_CONTEXT_NAME, LogSession.DEFAULT_CONTEXT_LOCATION,LogSession.DEFAULT_PROJECT_NAME);
			ld.initAllSessionEvents();
		}
		else ld = new LogData(LogData.DEFAULT_ME_ID, strName, LogSession.DEFAULT_CONTEXT_NAME, LogSession.DEFAULT_CONTEXT_LOCATION, LogSession.DEFAULT_PROJECT_NAME,strUser);
		
		for (Iterator iter = ld.getSessions().iterator(); iter.hasNext();)
		{
			LogSession session = (LogSession) iter.next();
			session.setParent(ld);
		}
		
		return ld;
	}
	

	private void initSessionEvents(final LogSession s)
	{
		if(s.ContextChanged == null) s.ContextChanged = new CustomEvent<String>();
		if(s.ProjectChanged == null) s.ProjectChanged = new CustomEvent<String>();
		if(this.SessionDataChanged == null) this.SessionDataChanged = new CustomEvent<LogSession>();
		s.ContextChanged.addListener(new CustomListener<String>(){
			public void onAction(String data)
			{
				SessionDataChanged.fire(s);
			}});
		s.ProjectChanged.addListener(new CustomListener<String>(){
			public void onAction(String data)
			{
				SessionDataChanged.fire(s);
			}});
		if(s.ActiveStateChanged == null) s.ActiveStateChanged = new CustomEvent<Boolean>();
		s.ActiveStateChanged.addListener(new CustomListener<Boolean>(){
			public void onAction(Boolean data)
			{
				if(data.booleanValue())
				{
					if((sessionActive != null) && (!sessionActive.equals(s)) ) sessionActive.setActive(false);
					sessionActive = s;
				}
			}});
	}
	

	public LogSession getActiveSession()
	{
		return sessionActive;
	}

	public void initAllSessionEvents()
	{
		for(LogSession s : this.Sessions)
		{
			this.initSessionEvents(s);
		}
	}
	
	/**
	 * Serializes the current log as XML to a file on disk.
	 *
	 */
	public static void saveToFile(LogData ld)
	{
		// Remove empty sessions
		Collection<LogSession> alSessions = ld.getSessions();
		Collection<LogSession> alSessionsToRemove = new ArrayList<LogSession>();
		for (Iterator iter = alSessions.iterator(); iter.hasNext();)
		{
			LogSession s = (LogSession) iter.next();
			if(s.getMessages().size() == 0) alSessionsToRemove.add(s);
		}
		for (Iterator iter = alSessionsToRemove.iterator(); iter.hasNext();)
		{
			LogSession s = (LogSession) iter.next();
			alSessions.remove(s);
		}		
		FileOutputStream output = null;
		OutputStreamWriter writer = null;
		try
		{
			ld.setLastSaved(Util.getDateFormatter().format(new Date()));
			XStream xStream = new XStream(new DomDriver());
			String strXml = xStream.toXML(ld);
			File file = getLogFilePath(ld.getYouUsername());
			output = new FileOutputStream(file.getAbsolutePath());
			writer = new OutputStreamWriter(output, Charset.forName("UTF-8"));
			writer.write(strXml);
			writer.close();
		}
		catch (Exception e)
		{
			GuiUtil.handleException(e);
		}
		finally
		{
			try
			{
				if (writer != null) writer.close();
				if (output != null) output.close();
			}
			catch (Exception e)
			{
				GuiUtil.handleException(e);
			}
		}
	}
	
	/**
	 * Serializes the current log to valid XML
	 */
	public static String getCurrentLogAsXml(LogData ld)
	{
		XStream xStream = new XStream(new DomDriver());
		return xStream.toXML(ld);
	}
	
	/**
	 * Deserializes a XML log file to objects
	 * @param strLogFile The log file.
	 * @return LogData object with all entries from the logfile
	 * @throws IOException 
	 */
	private static LogData loadFromFile(String strLogFile) throws IOException
	{
		FileInputStream input = null;
		InputStreamReader reader = null;
		XStream xStream = new XStream(new DomDriver());
		input = new FileInputStream(strLogFile);
		reader = new InputStreamReader(input, Charset.forName("UTF-8"));
		LogData logFromXml = (LogData)xStream.fromXML(reader);
		if (reader != null) reader.close();
		if (input != null) input.close();
		return logFromXml;
	}
	
	/**
	 * Adds incoming message to log.
	 * @param strName The name of the user that sent the message.
	 * @param strBody The message body.
	 * @param strContext The context of the message.
	 * @param strMoreInfo 
	 * @param strLocation 
	 */
	public void handleMessage(String strName, String strBody, final String strContext, final String strProject, final String strLocation, final String strMoreInfo) throws UnsupportedEncodingException
	{
		final LogSession session = this.getSessionByContext(strContext,strProject);
		session.addMessage(strBody,true,Util.getDateFormatter().format(new Date()));
		session.setContextLocation(strLocation);
		session.setMoreInfo(strMoreInfo);
	}
	
	/**
	 * Sends a message over the XMPP connection. Adds to log.
	 */
	public void sendMessage(String strText) throws XMPPException, UnsupportedEncodingException
	{
		LogSession sess = this.getActiveSession();
		String strContext = sess.getContext();
		String strProject = sess.getProject();
		String strLocation = sess.getContextLocation();
		String strMoreInfo = sess.getMoreInfo();
		if(strContext == null) strContext = LogSession.DEFAULT_CONTEXT_NAME;
		if(strProject == null) strProject = LogSession.DEFAULT_PROJECT_NAME;
		if(strLocation == null) strLocation = LogSession.DEFAULT_CONTEXT_LOCATION;
		if(strMoreInfo == null) strMoreInfo = "";
		Message m = new Message();
		m.setBody(strText);
		m.setProperty(SimpleChatPlugin.CONTEXT_PROPERTY_ID,strContext);
		m.setProperty(SimpleChatPlugin.PROJECT_PROPERTY_ID, strProject);
		m.setProperty(SimpleChatPlugin.CONTEXT_LOCATION_PROPERTY_ID, strLocation);
		m.setProperty(SimpleChatPlugin.MORE_INFO_PROPERTY_ID, strMoreInfo);
		if(chat == null) chat = SimpleChatPlugin.getDefault().getConnection().createChat(this.YouUsername);
		chat.sendMessage(m);
		sess.addMessage(strText,false,Util.getDateFormatter().format(new Date()));
	}
	
	public void dispose(boolean bCloseGui)
	{
		chat = null;
		saveToFile(this);
		nullActiveSession();
		if(this.Disposed == null) this.Disposed = new CustomEvent<Boolean>();
		this.Disposed.fire(new Boolean(bCloseGui));
	}

	public void nullActiveSession()
	{
		if(this.sessionActive != null)
		{
			this.sessionActive.setActive(false);
			this.sessionActive = null;
		}
	}

	public void setContactNode(TreeItem nodeContact)
	{
		this.nodeContact = nodeContact;
	}
	public TreeItem getContactNode()
	{
		return this.nodeContact;
	}

	public Collection<Project> getProjects()
	{
		Hashtable<String,Project> ht = new Hashtable<String,Project>();
		for(LogSession s : Sessions)
		{
			String strPrj = s.getProject();
			if(Util.isStringEmpty(strPrj, LogSession.DEFAULT_PROJECT_NAME)) strPrj = LogSession.DEFAULT_PROJECT_NAME;
			if(ht.get(strPrj) == null)
			{
				Project p = new Project(this);
				p.setCommonName(strPrj);
				ht.put(strPrj, p);
			}
		}
		return ht.values();
	}

	public Collection<LogSession> getSessionsByProject(Project p)
	{
		ArrayList<LogSession> al = new ArrayList<LogSession>();
		for(LogSession s : this.getSessions())
		{
			String strPrj = s.getProject();
			if(Util.isStringEmpty(strPrj, LogSession.DEFAULT_PROJECT_NAME)) strPrj = LogSession.DEFAULT_PROJECT_NAME;
			if(strPrj.equalsIgnoreCase(p.getCommonName())) al.add(s);
		}
		return al;
	}
}