package no.jabber.eclipseplugin.serialization;

import java.util.*;

import org.eclipse.swt.widgets.Display;

import no.jabber.eclipseplugin.Util;
import no.jabber.eclipseplugin.Util.CustomEvent;

/**
 * Represents a single chat session. I.e. a collection of LogEntry objects which all have the same context. Used for serializing and deserializing chat log.
 */
@SuppressWarnings("unused")
public class LogSession
{
	public transient static String DEFAULT_PROJECT_NAME = "(none)";
	public transient static String DEFAULT_CONTEXT_NAME = "default";
	public transient static String DEFAULT_CONTEXT_LOCATION = "(none)";
	private transient LogData parent;
	private transient boolean bIsActive;
	private String Context;
	private String Project;
	private String ContextLocation;
	private String MoreInfo;
	private ArrayList<LogEntry> Messages = new ArrayList<LogEntry>();
	public transient Util.CustomEvent<String> ContextChanged = new CustomEvent<String>();
	public transient Util.CustomEvent<String> ContextLocationChanged = new CustomEvent<String>();
	public transient Util.CustomEvent<String> ProjectChanged = new CustomEvent<String>();
	public transient Util.CustomEvent<String> MoreInfoChanged = new CustomEvent<String>();
	public transient Util.CustomEvent<LogEntry> MessageAdded = new CustomEvent<LogEntry>();
	public transient Util.CustomEvent<Boolean> ActiveStateChanged = new CustomEvent<Boolean>();
	

	public LogSession(LogData parent, String strContext, String strContextLocation, String strProject)
	{
		this.parent = parent;
		this.Context = strContext;
		this.ContextLocation = strContextLocation;
		this.Project = strProject;
		this.ContextChanged.fire(this.Context);
		this.ProjectChanged.fire(this.Project);
		this.ContextLocationChanged.fire(this.ContextLocation);
	}

	public String getLatestTimestamp()
	{
		if(this.Messages.size() > 0)
		{
			return this.Messages.get(Messages.size() - 1).getTimestamp();
		}
		else return "";
	}
	
	public ArrayList<LogEntry> getMessages()
	{
		return this.Messages;
	}

	public void setMessages(ArrayList<LogEntry> messages)
	{
		Messages = messages;
	}

	public LogData getParent()
	{
		return parent;
	}

	public void setParent(LogData parent)
	{
		this.parent = parent;
	}

	public LogEntry addMessage(String strMsg, boolean bIncoming, String dtTimestamp)
	{
		// In case of URLs; add a whitespace at the end to make formatting of URLs correct
//		for (int i = 0; i < Util.RX_HYPERLINKS.length; i++)
//		{
//			strMsg = strMsg.replaceAll(Util.RX_HYPERLINKS[i], "$0 ");
//		}

		String strFrom = (bIncoming ? parent.getYou() : parent.getMe());
		String strTo = (bIncoming ? parent.getMe() : parent.getYou());
		final LogEntry entry = new LogEntry(strFrom, strTo, strMsg, dtTimestamp,this);
		this.Messages.add(entry);
		if(this.MessageAdded == null) this.MessageAdded = new CustomEvent<LogEntry>();
		Display.getDefault().syncExec(new Runnable(){

			public void run()
			{
				MessageAdded.fire(entry);
			}});
		return entry;
	}

	public String getContext()
	{
		return this.Context;
	}

	public void setContext(String context)
	{
		this.Context = context;
		this.ContextChanged.fire(this.Context);
	}

	public String getContextLocation()
	{
		return this.ContextLocation;
	}

	public void setContextLocation(String contextLocation)
	{
		this.ContextLocation = contextLocation;
		this.ContextLocationChanged.fire(this.ContextLocation);
	}

	public String getProject()
	{
		return this.Project;
	}

	public void setProject(String project)
	{
		this.Project = project;
		this.ProjectChanged.fire(this.Project);
	}

	public boolean isActive()
	{
		return this.bIsActive;
	}

	public void setActive(boolean isActive)
	{
		this.bIsActive = isActive;
		if(this.ActiveStateChanged == null) this.ActiveStateChanged = new CustomEvent<Boolean>();
		this.ActiveStateChanged.fire(isActive);
	}
	
	public String toString()
	{
		return this.Context;
	}

	public String getMoreInfo()
	{
		return this.MoreInfo;
	}

	public void setMoreInfo(String moreInfo)
	{
		//moreInfo = XmlUtil.stringToHTMLString(moreInfo);
		this.MoreInfo = moreInfo;
		if(this.MoreInfoChanged == null) this.MoreInfoChanged = new CustomEvent<String>();
		this.MoreInfoChanged.fire(this.MoreInfo);
	}

	public String getMessagesAsText()
	{
		StringBuilder sb = new StringBuilder();
		for(LogEntry entry : this.getMessages())
		{
			sb.append(entry.getFrom());
			sb.append(": ");
			sb.append(entry.getMsg());
		}
		return sb.toString();
	}
}